# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.4.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if getattr(globals().get("__spec__"), "parent", None) or __package__ or "." in __name__:
    from . import _ogr
else:
    import _ogr

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


wkb25DBit = _ogr.wkb25DBit

wkb25Bit = _ogr.wkb25Bit

wkbUnknown = _ogr.wkbUnknown

wkbPoint = _ogr.wkbPoint

wkbLineString = _ogr.wkbLineString

wkbPolygon = _ogr.wkbPolygon

wkbMultiPoint = _ogr.wkbMultiPoint

wkbMultiLineString = _ogr.wkbMultiLineString

wkbMultiPolygon = _ogr.wkbMultiPolygon

wkbGeometryCollection = _ogr.wkbGeometryCollection

wkbCircularString = _ogr.wkbCircularString

wkbCompoundCurve = _ogr.wkbCompoundCurve

wkbCurvePolygon = _ogr.wkbCurvePolygon

wkbMultiCurve = _ogr.wkbMultiCurve

wkbMultiSurface = _ogr.wkbMultiSurface

wkbCurve = _ogr.wkbCurve

wkbSurface = _ogr.wkbSurface

wkbPolyhedralSurface = _ogr.wkbPolyhedralSurface

wkbTIN = _ogr.wkbTIN

wkbTriangle = _ogr.wkbTriangle

wkbNone = _ogr.wkbNone

wkbLinearRing = _ogr.wkbLinearRing

wkbCircularStringZ = _ogr.wkbCircularStringZ

wkbCompoundCurveZ = _ogr.wkbCompoundCurveZ

wkbCurvePolygonZ = _ogr.wkbCurvePolygonZ

wkbMultiCurveZ = _ogr.wkbMultiCurveZ

wkbMultiSurfaceZ = _ogr.wkbMultiSurfaceZ

wkbCurveZ = _ogr.wkbCurveZ

wkbSurfaceZ = _ogr.wkbSurfaceZ

wkbPolyhedralSurfaceZ = _ogr.wkbPolyhedralSurfaceZ

wkbTINZ = _ogr.wkbTINZ

wkbTriangleZ = _ogr.wkbTriangleZ

wkbPointM = _ogr.wkbPointM

wkbLineStringM = _ogr.wkbLineStringM

wkbPolygonM = _ogr.wkbPolygonM

wkbMultiPointM = _ogr.wkbMultiPointM

wkbMultiLineStringM = _ogr.wkbMultiLineStringM

wkbMultiPolygonM = _ogr.wkbMultiPolygonM

wkbGeometryCollectionM = _ogr.wkbGeometryCollectionM

wkbCircularStringM = _ogr.wkbCircularStringM

wkbCompoundCurveM = _ogr.wkbCompoundCurveM

wkbCurvePolygonM = _ogr.wkbCurvePolygonM

wkbMultiCurveM = _ogr.wkbMultiCurveM

wkbMultiSurfaceM = _ogr.wkbMultiSurfaceM

wkbCurveM = _ogr.wkbCurveM

wkbSurfaceM = _ogr.wkbSurfaceM

wkbPolyhedralSurfaceM = _ogr.wkbPolyhedralSurfaceM

wkbTINM = _ogr.wkbTINM

wkbTriangleM = _ogr.wkbTriangleM

wkbPointZM = _ogr.wkbPointZM

wkbLineStringZM = _ogr.wkbLineStringZM

wkbPolygonZM = _ogr.wkbPolygonZM

wkbMultiPointZM = _ogr.wkbMultiPointZM

wkbMultiLineStringZM = _ogr.wkbMultiLineStringZM

wkbMultiPolygonZM = _ogr.wkbMultiPolygonZM

wkbGeometryCollectionZM = _ogr.wkbGeometryCollectionZM

wkbCircularStringZM = _ogr.wkbCircularStringZM

wkbCompoundCurveZM = _ogr.wkbCompoundCurveZM

wkbCurvePolygonZM = _ogr.wkbCurvePolygonZM

wkbMultiCurveZM = _ogr.wkbMultiCurveZM

wkbMultiSurfaceZM = _ogr.wkbMultiSurfaceZM

wkbCurveZM = _ogr.wkbCurveZM

wkbSurfaceZM = _ogr.wkbSurfaceZM

wkbPolyhedralSurfaceZM = _ogr.wkbPolyhedralSurfaceZM

wkbTINZM = _ogr.wkbTINZM

wkbTriangleZM = _ogr.wkbTriangleZM

wkbPoint25D = _ogr.wkbPoint25D

wkbLineString25D = _ogr.wkbLineString25D

wkbPolygon25D = _ogr.wkbPolygon25D

wkbMultiPoint25D = _ogr.wkbMultiPoint25D

wkbMultiLineString25D = _ogr.wkbMultiLineString25D

wkbMultiPolygon25D = _ogr.wkbMultiPolygon25D

wkbGeometryCollection25D = _ogr.wkbGeometryCollection25D

OFTInteger = _ogr.OFTInteger

OFTIntegerList = _ogr.OFTIntegerList

OFTReal = _ogr.OFTReal

OFTRealList = _ogr.OFTRealList

OFTString = _ogr.OFTString

OFTStringList = _ogr.OFTStringList

OFTWideString = _ogr.OFTWideString

OFTWideStringList = _ogr.OFTWideStringList

OFTBinary = _ogr.OFTBinary

OFTDate = _ogr.OFTDate

OFTTime = _ogr.OFTTime

OFTDateTime = _ogr.OFTDateTime

OFTInteger64 = _ogr.OFTInteger64

OFTInteger64List = _ogr.OFTInteger64List

OFSTNone = _ogr.OFSTNone

OFSTBoolean = _ogr.OFSTBoolean

OFSTInt16 = _ogr.OFSTInt16

OFSTFloat32 = _ogr.OFSTFloat32

OFSTJSON = _ogr.OFSTJSON

OFSTUUID = _ogr.OFSTUUID

OJUndefined = _ogr.OJUndefined

OJLeft = _ogr.OJLeft

OJRight = _ogr.OJRight

OFDT_CODED = _ogr.OFDT_CODED

OFDT_RANGE = _ogr.OFDT_RANGE

OFDT_GLOB = _ogr.OFDT_GLOB

OFDSP_DEFAULT_VALUE = _ogr.OFDSP_DEFAULT_VALUE

OFDSP_DUPLICATE = _ogr.OFDSP_DUPLICATE

OFDSP_GEOMETRY_RATIO = _ogr.OFDSP_GEOMETRY_RATIO

OFDMP_DEFAULT_VALUE = _ogr.OFDMP_DEFAULT_VALUE

OFDMP_SUM = _ogr.OFDMP_SUM

OFDMP_GEOMETRY_WEIGHTED = _ogr.OFDMP_GEOMETRY_WEIGHTED

wkbXDR = _ogr.wkbXDR

wkbNDR = _ogr.wkbNDR

NullFID = _ogr.NullFID

ALTER_NAME_FLAG = _ogr.ALTER_NAME_FLAG

ALTER_TYPE_FLAG = _ogr.ALTER_TYPE_FLAG

ALTER_WIDTH_PRECISION_FLAG = _ogr.ALTER_WIDTH_PRECISION_FLAG

ALTER_NULLABLE_FLAG = _ogr.ALTER_NULLABLE_FLAG

ALTER__FLAG = _ogr.ALTER__FLAG

ALTER_DEFAULT_FLAG = _ogr.ALTER_DEFAULT_FLAG

ALTER_UNIQUE_FLAG = _ogr.ALTER_UNIQUE_FLAG

ALTER_DOMAIN_FLAG = _ogr.ALTER_DOMAIN_FLAG

ALTER_ALTERNATIVE_NAME_FLAG = _ogr.ALTER_ALTERNATIVE_NAME_FLAG

ALTER_COMMENT_FLAG = _ogr.ALTER_COMMENT_FLAG

ALTER_ALL_FLAG = _ogr.ALTER_ALL_FLAG

ALTER_GEOM_FIELD_DEFN_NAME_FLAG = _ogr.ALTER_GEOM_FIELD_DEFN_NAME_FLAG

ALTER_GEOM_FIELD_DEFN_TYPE_FLAG = _ogr.ALTER_GEOM_FIELD_DEFN_TYPE_FLAG

ALTER_GEOM_FIELD_DEFN_NULLABLE_FLAG = _ogr.ALTER_GEOM_FIELD_DEFN_NULLABLE_FLAG

ALTER_GEOM_FIELD_DEFN_SRS_FLAG = _ogr.ALTER_GEOM_FIELD_DEFN_SRS_FLAG

ALTER_GEOM_FIELD_DEFN_SRS_COORD_EPOCH_FLAG = _ogr.ALTER_GEOM_FIELD_DEFN_SRS_COORD_EPOCH_FLAG

ALTER_GEOM_FIELD_DEFN_ALL_FLAG = _ogr.ALTER_GEOM_FIELD_DEFN_ALL_FLAG

F_VAL_NULL = _ogr.F_VAL_NULL

F_VAL_GEOM_TYPE = _ogr.F_VAL_GEOM_TYPE

F_VAL_WIDTH = _ogr.F_VAL_WIDTH

F_VAL_ALLOW_NULL_WHEN_DEFAULT = _ogr.F_VAL_ALLOW_NULL_WHEN_DEFAULT

F_VAL_ALL = _ogr.F_VAL_ALL

TZFLAG_UNKNOWN = _ogr.TZFLAG_UNKNOWN

TZFLAG_LOCALTIME = _ogr.TZFLAG_LOCALTIME

TZFLAG_MIXED_TZ = _ogr.TZFLAG_MIXED_TZ

TZFLAG_UTC = _ogr.TZFLAG_UTC

GGT_COUNT_NOT_NEEDED = _ogr.GGT_COUNT_NOT_NEEDED

GGT_STOP_IF_MIXED = _ogr.GGT_STOP_IF_MIXED

GGT_GEOMCOLLECTIONZ_TINZ = _ogr.GGT_GEOMCOLLECTIONZ_TINZ

OLCRandomRead = _ogr.OLCRandomRead

OLCSequentialWrite = _ogr.OLCSequentialWrite

OLCRandomWrite = _ogr.OLCRandomWrite

OLCFastSpatialFilter = _ogr.OLCFastSpatialFilter

OLCFastFeatureCount = _ogr.OLCFastFeatureCount

OLCFastGetExtent = _ogr.OLCFastGetExtent

OLCFastGetExtent3D = _ogr.OLCFastGetExtent3D

OLCCreateField = _ogr.OLCCreateField

OLCDeleteField = _ogr.OLCDeleteField

OLCReorderFields = _ogr.OLCReorderFields

OLCAlterFieldDefn = _ogr.OLCAlterFieldDefn

OLCAlterGeomFieldDefn = _ogr.OLCAlterGeomFieldDefn

OLCTransactions = _ogr.OLCTransactions

OLCDeleteFeature = _ogr.OLCDeleteFeature

OLCUpsertFeature = _ogr.OLCUpsertFeature

OLCUpdateFeature = _ogr.OLCUpdateFeature

OLCFastSetNextByIndex = _ogr.OLCFastSetNextByIndex

OLCStringsAsUTF8 = _ogr.OLCStringsAsUTF8

OLCIgnoreFields = _ogr.OLCIgnoreFields

OLCCreateGeomField = _ogr.OLCCreateGeomField

OLCCurveGeometries = _ogr.OLCCurveGeometries

OLCMeasuredGeometries = _ogr.OLCMeasuredGeometries

OLCZGeometries = _ogr.OLCZGeometries

OLCRename = _ogr.OLCRename

OLCFastGetArrowStream = _ogr.OLCFastGetArrowStream

OLCFastWriteArrowBatch = _ogr.OLCFastWriteArrowBatch

ODsCCreateLayer = _ogr.ODsCCreateLayer

ODsCDeleteLayer = _ogr.ODsCDeleteLayer

ODsCCreateGeomFieldAfterCreateLayer = _ogr.ODsCCreateGeomFieldAfterCreateLayer

ODsCCurveGeometries = _ogr.ODsCCurveGeometries

ODsCTransactions = _ogr.ODsCTransactions

ODsCEmulatedTransactions = _ogr.ODsCEmulatedTransactions

ODsCMeasuredGeometries = _ogr.ODsCMeasuredGeometries

ODsCZGeometries = _ogr.ODsCZGeometries

ODsCRandomLayerRead = _ogr.ODsCRandomLayerRead

ODsCRandomLayerWrite = _ogr.ODsCRandomLayerWrite

ODsCAddFieldDomain = _ogr.ODsCAddFieldDomain

ODsCDeleteFieldDomain = _ogr.ODsCDeleteFieldDomain

ODsCUpdateFieldDomain = _ogr.ODsCUpdateFieldDomain

ODsCAddRelationship = _ogr.ODsCAddRelationship

ODsCDeleteRelationship = _ogr.ODsCDeleteRelationship

ODsCUpdateRelationship = _ogr.ODsCUpdateRelationship

ODrCCreateDataSource = _ogr.ODrCCreateDataSource

ODrCDeleteDataSource = _ogr.ODrCDeleteDataSource

OLMD_FID64 = _ogr.OLMD_FID64

GEOS_PREC_NO_TOPO = _ogr.GEOS_PREC_NO_TOPO

GEOS_PREC_KEEP_COLLAPSED = _ogr.GEOS_PREC_KEEP_COLLAPSED

OGRERR_NONE = _ogr.OGRERR_NONE

OGRERR_NOT_ENOUGH_DATA = _ogr.OGRERR_NOT_ENOUGH_DATA

OGRERR_NOT_ENOUGH_MEMORY = _ogr.OGRERR_NOT_ENOUGH_MEMORY

OGRERR_UNSUPPORTED_GEOMETRY_TYPE = _ogr.OGRERR_UNSUPPORTED_GEOMETRY_TYPE

OGRERR_UNSUPPORTED_OPERATION = _ogr.OGRERR_UNSUPPORTED_OPERATION

OGRERR_CORRUPT_DATA = _ogr.OGRERR_CORRUPT_DATA

OGRERR_FAILURE = _ogr.OGRERR_FAILURE

OGRERR_UNSUPPORTED_SRS = _ogr.OGRERR_UNSUPPORTED_SRS

OGRERR_INVALID_HANDLE = _ogr.OGRERR_INVALID_HANDLE

OGRERR_NON_EXISTING_FEATURE = _ogr.OGRERR_NON_EXISTING_FEATURE


def GetUseExceptions(*args):
    r"""GetUseExceptions() -> int"""
    return _ogr.GetUseExceptions(*args)

def _GetExceptionsLocal(*args):
    r"""_GetExceptionsLocal() -> int"""
    return _ogr._GetExceptionsLocal(*args)

def _SetExceptionsLocal(*args):
    r"""_SetExceptionsLocal(bVal)"""
    return _ogr._SetExceptionsLocal(*args)

def _UseExceptions(*args):
    r"""_UseExceptions()"""
    return _ogr._UseExceptions(*args)

def _DontUseExceptions(*args):
    r"""_DontUseExceptions()"""
    return _ogr._DontUseExceptions(*args)

def _UserHasSpecifiedIfUsingExceptions(*args):
    r"""_UserHasSpecifiedIfUsingExceptions() -> int"""
    return _ogr._UserHasSpecifiedIfUsingExceptions(*args)

class ExceptionMgr:
    """
    Context manager to manage Python Exception state
    for GDAL/OGR/OSR/GNM.

    Separate exception state is maintained for each
    module (gdal, ogr, etc), and this class appears independently
    in all of them. This is built in top of calls to the older
    UseExceptions()/DontUseExceptions() functions.

    Example::

        >>> print(gdal.GetUseExceptions())
        0
        >>> with gdal.ExceptionMgr():
        ...     # Exceptions are now in use
        ...     print(gdal.GetUseExceptions())
        1
        >>>
        >>> # Exception state has now been restored
        >>> print(gdal.GetUseExceptions())
        0

    """
    def __init__(self, useExceptions=True):
        """
        Save whether or not this context will be using exceptions
        """
        self.requestedUseExceptions = useExceptions

    def __enter__(self):
        """
        On context entry, save the current GDAL exception state, and
        set it to the state requested for the context

        """
        self.currentUseExceptions = _GetExceptionsLocal()
        _SetExceptionsLocal(self.requestedUseExceptions)
        if ExceptionMgr.__module__ == "osgeo.gdal":
            try:
                from . import gdal_array
            except ImportError:
                gdal_array = None
            if gdal_array:
                gdal_array._SetExceptionsLocal(self.requestedUseExceptions)

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        On exit, restore the GDAL/OGR/OSR/GNM exception state which was
        current on entry to the context
        """
        _SetExceptionsLocal(self.currentUseExceptions)
        if ExceptionMgr.__module__ == "osgeo.gdal":
            try:
                from . import gdal_array
            except ImportError:
                gdal_array = None
            if gdal_array:
                gdal_array._SetExceptionsLocal(self.currentUseExceptions)




import importlib.util

def _has_gdal_array():
    return importlib.util.find_spec("osgeo.gdal_array") and importlib.util.find_spec("numpy")

def UseExceptions():
    """ Enable exceptions in all GDAL related modules (osgeo.gdal, osgeo.ogr, osgeo.osr, osgeo.gnm).
        Note: prior to GDAL 3.7, this only affected the calling module"""

    try:
        from . import gdal
        gdal._UseExceptions()
    except ImportError:
        pass

    if _has_gdal_array():
        try:
            from . import gdal_array
            gdal_array._UseExceptions()
        except ImportError:
            pass
    try:
        from . import ogr
        ogr._UseExceptions()
    except ImportError:
        pass

    try:
        from . import osr
        osr._UseExceptions()
    except ImportError:
        pass

    try:
        from . import gnm
        gnm._UseExceptions()
    except ImportError:
        pass

def DontUseExceptions():
    """ Disable exceptions in all GDAL related modules (osgeo.gdal, osgeo.ogr, osgeo.osr, osgeo.gnm).
        Note: prior to GDAL 3.7, this only affected the calling module"""

    try:
        from . import gdal
        gdal._DontUseExceptions()
    except ImportError:
        pass

    if _has_gdal_array():
        try:
            from . import gdal_array
            gdal_array._DontUseExceptions()
        except ImportError:
            pass

    try:
        from . import ogr
        ogr._DontUseExceptions()
    except ImportError:
        pass

    try:
        from . import osr
        osr._DontUseExceptions()
    except ImportError:
        pass

    try:
        from . import gnm
        gnm._DontUseExceptions()
    except ImportError:
        pass




hasWarnedAboutUserHasNotSpecifiedIfUsingExceptions = False

def _WarnIfUserHasNotSpecifiedIfUsingExceptions():
    from . import gdal
    if not hasattr(gdal, "hasWarnedAboutUserHasNotSpecifiedIfUsingExceptions") and not _UserHasSpecifiedIfUsingExceptions():
        gdal.hasWarnedAboutUserHasNotSpecifiedIfUsingExceptions = True
        import warnings
        warnings.warn(
            "Neither ogr.UseExceptions() nor ogr.DontUseExceptions() has been explicitly called. " +
            "In GDAL 4.0, exceptions will be enabled by default.", FutureWarning)

from . import osr
class MajorObject(object):
    r"""Proxy of C++ GDALMajorObjectShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def GetDescription(self, *args):
        r"""GetDescription(self) -> char const *"""
        return _ogr.MajorObject_GetDescription(self, *args)

    def SetDescription(self, *args):
        r"""SetDescription(self, pszNewDesc)"""
        return _ogr.MajorObject_SetDescription(self, *args)

    def GetMetadataDomainList(self, *args):
        r"""GetMetadataDomainList(self) -> char **"""
        return _ogr.MajorObject_GetMetadataDomainList(self, *args)

    def GetMetadata_Dict(self, *args):
        r"""GetMetadata_Dict(self, pszDomain="") -> char **"""
        return _ogr.MajorObject_GetMetadata_Dict(self, *args)

    def GetMetadata_List(self, *args):
        r"""GetMetadata_List(self, pszDomain="") -> char **"""
        return _ogr.MajorObject_GetMetadata_List(self, *args)

    def SetMetadata(self, *args):
        r"""
        SetMetadata(self, papszMetadata, pszDomain="") -> CPLErr
        SetMetadata(self, pszMetadataString, pszDomain="") -> CPLErr
        """
        return _ogr.MajorObject_SetMetadata(self, *args)

    def GetMetadataItem(self, *args):
        r"""GetMetadataItem(self, pszName, pszDomain="") -> char const *"""
        return _ogr.MajorObject_GetMetadataItem(self, *args)

    def SetMetadataItem(self, *args):
        r"""SetMetadataItem(self, pszName, pszValue, pszDomain="") -> CPLErr"""
        return _ogr.MajorObject_SetMetadataItem(self, *args)

    def GetMetadata(self, domain=''):
      if domain[:4] == 'xml:':
        return self.GetMetadata_List(domain)
      return self.GetMetadata_Dict(domain)


# Register MajorObject in _ogr:
_ogr.MajorObject_swigregister(MajorObject)

def GetFieldTypeByName(*args):
    r"""GetFieldTypeByName(typeName) -> OGRFieldType"""
    return _ogr.GetFieldTypeByName(*args)

def GetFieldSubtypeByName(*args):
    r"""GetFieldSubtypeByName(subTypeName) -> OGRFieldSubType"""
    return _ogr.GetFieldSubtypeByName(*args)

def GetGEOSVersionMajor(*args):
    r"""GetGEOSVersionMajor() -> int"""
    return _ogr.GetGEOSVersionMajor(*args)

def GetGEOSVersionMinor(*args):
    r"""GetGEOSVersionMinor() -> int"""
    return _ogr.GetGEOSVersionMinor(*args)

def GetGEOSVersionMicro(*args):
    r"""GetGEOSVersionMicro() -> int"""
    return _ogr.GetGEOSVersionMicro(*args)
class StyleTable(object):
    r"""Proxy of C++ OGRStyleTableShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(self) -> StyleTable"""
        _ogr.StyleTable_swiginit(self, _ogr.new_StyleTable(*args))
    __swig_destroy__ = _ogr.delete_StyleTable

    def AddStyle(self, *args):
        r"""AddStyle(self, pszName, pszStyleString) -> int"""
        return _ogr.StyleTable_AddStyle(self, *args)

    def LoadStyleTable(self, *args):
        r"""LoadStyleTable(self, utf8_string) -> int"""
        return _ogr.StyleTable_LoadStyleTable(self, *args)

    def SaveStyleTable(self, *args):
        r"""SaveStyleTable(self, utf8_string) -> int"""
        return _ogr.StyleTable_SaveStyleTable(self, *args)

    def Find(self, *args):
        r"""Find(self, pszName) -> char const *"""
        return _ogr.StyleTable_Find(self, *args)

    def ResetStyleStringReading(self, *args):
        r"""ResetStyleStringReading(self)"""
        return _ogr.StyleTable_ResetStyleStringReading(self, *args)

    def GetNextStyle(self, *args):
        r"""GetNextStyle(self) -> char const *"""
        return _ogr.StyleTable_GetNextStyle(self, *args)

    def GetLastStyleName(self, *args):
        r"""GetLastStyleName(self) -> char const *"""
        return _ogr.StyleTable_GetLastStyleName(self, *args)

# Register StyleTable in _ogr:
_ogr.StyleTable_swigregister(StyleTable)
class ArrowArray(object):
    r"""Proxy of C++ ArrowArray class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(self) -> ArrowArray"""
        _ogr.ArrowArray_swiginit(self, _ogr.new_ArrowArray(*args))
    __swig_destroy__ = _ogr.delete_ArrowArray

    def _getPtr(self, *args):
        r"""_getPtr(self) -> VoidPtrAsLong"""
        return _ogr.ArrowArray__getPtr(self, *args)

    def GetChildrenCount(self, *args):
        r"""GetChildrenCount(self) -> GIntBig"""
        return _ogr.ArrowArray_GetChildrenCount(self, *args)

    def GetLength(self, *args):
        r"""GetLength(self) -> GIntBig"""
        return _ogr.ArrowArray_GetLength(self, *args)

# Register ArrowArray in _ogr:
_ogr.ArrowArray_swigregister(ArrowArray)
class ArrowSchema(object):
    r"""Proxy of C++ ArrowSchema class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(self) -> ArrowSchema"""
        _ogr.ArrowSchema_swiginit(self, _ogr.new_ArrowSchema(*args))
    __swig_destroy__ = _ogr.delete_ArrowSchema

    def _getPtr(self, *args):
        r"""_getPtr(self) -> VoidPtrAsLong"""
        return _ogr.ArrowSchema__getPtr(self, *args)

    def GetName(self, *args):
        r"""GetName(self) -> char const *"""
        return _ogr.ArrowSchema_GetName(self, *args)

    def GetChildrenCount(self, *args):
        r"""GetChildrenCount(self) -> GIntBig"""
        return _ogr.ArrowSchema_GetChildrenCount(self, *args)

    def GetChild(self, *args):
        r"""GetChild(self, iChild) -> ArrowSchema"""
        return _ogr.ArrowSchema_GetChild(self, *args)

# Register ArrowSchema in _ogr:
_ogr.ArrowSchema_swigregister(ArrowSchema)
class ArrowArrayStream(object):
    r"""Proxy of C++ ArrowArrayStream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_ArrowArrayStream

    def GetSchema(self, *args):
        r"""GetSchema(self) -> ArrowSchema"""
        return _ogr.ArrowArrayStream_GetSchema(self, *args)

    def GetNextRecordBatch(self, *args):
        r"""GetNextRecordBatch(self, options=None) -> ArrowArray"""
        return _ogr.ArrowArrayStream_GetNextRecordBatch(self, *args)

# Register ArrowArrayStream in _ogr:
_ogr.ArrowArrayStream_swigregister(ArrowArrayStream)
class Layer(MajorObject):
    r"""Proxy of C++ OGRLayerShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def GetDataset(self, *args):
        r"""
        GetDataset(self) -> GDALDatasetShadow *

        Return the dataset associated with this layer.

        For more details: :cpp:func:`OGR_L_GetDataset`

        Returns
        -------
        Dataset or None
            A dataset if successful, or ``None`` on failure.

        """
        return _ogr.Layer_GetDataset(self, *args)

    def Rename(self, *args):
        r"""Rename(self, new_name) -> OGRErr"""
        return _ogr.Layer_Rename(self, *args)

    def GetRefCount(self, *args):
        r"""
        GetRefCount(self) -> int

        For more details: :cpp:func:`OGR_L_GetRefCount`

        """
        return _ogr.Layer_GetRefCount(self, *args)

    def SetSpatialFilter(self, *args):
        r"""
        SetSpatialFilter(self, filter)
        SetSpatialFilter(self, iGeomField, filter)

        Set a new spatial filter.

        For more details:

        - :cpp:func:`OGR_L_SetSpatialFilter`
        - :cpp:func:`OGR_L_SetSpatialFilterEx`

        Parameters
        ----------
        iGeomField : int, optional
            index of the geometry field on which the spatial filter operates.
        filter : Geometry
            The geometry to use as a filtering region. None may
            be passed indicating that the current spatial filter should be
            cleared, but no new one instituted.

        """
        return _ogr.Layer_SetSpatialFilter(self, *args)

    def SetSpatialFilterRect(self, *args):
        r"""
        SetSpatialFilterRect(self, minx, miny, maxx, maxy)
        SetSpatialFilterRect(self, iGeomField, minx, miny, maxx, maxy)

        Set a new rectangular spatial filter.

        For more details:

        - :cpp:func:`OGR_L_SetSpatialFilterRect`
        - :cpp:func:`OGR_L_SetSpatialFilterRectEx`

        Parameters
        ----------
        iGeomField : int, optional
            index of the geometry field on which the spatial filter operates.
        minx : float
            the minimum X coordinate for the rectangular region.
        miny : float
            the minimum Y coordinate for the rectangular region.
        maxx : float
            the maximum X coordinate for the rectangular region.
        maxy : float
            the maximum Y coordinate for the rectangular region.

        """
        return _ogr.Layer_SetSpatialFilterRect(self, *args)

    def GetSpatialFilter(self, *args):
        r"""
        GetSpatialFilter(self) -> Geometry

        This function returns the current spatial filter for this layer.

        For more details: :cpp:func:`OGR_L_GetSpatialFilter`

        Returns
        -------
        Geometry
            The spatial filter geometry.

        """
        return _ogr.Layer_GetSpatialFilter(self, *args)

    def SetAttributeFilter(self, *args):
        r"""
        SetAttributeFilter(self, filter_string) -> OGRErr

        Set a new attribute query.

        For more details: :cpp:func:`OGR_L_SetAttributeFilter`

        Parameters
        ----------
        filter_string : str
            query in restricted SQL WHERE format, or None to clear the
            current query.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` if successfully installed,
            or an error code if the query expression is in error,
            or some other failure occurs.

        """
        return _ogr.Layer_SetAttributeFilter(self, *args)

    def GetAttributeFilter(self, *args):
        r"""GetAttributeFilter(self) -> char const *"""
        return _ogr.Layer_GetAttributeFilter(self, *args)

    def ResetReading(self, *args):
        r"""
        ResetReading(self)

        Reset feature reading to start on the first feature.

        For more details: :cpp:func:`OGR_L_ResetReading`

        """
        return _ogr.Layer_ResetReading(self, *args)

    def GetName(self, *args):
        r"""
        GetName(self) -> char const *

        Return the layer name.

        For more details: :cpp:func:`OGR_L_GetName`

        Returns
        -------
        str
            The layer name

        """
        return _ogr.Layer_GetName(self, *args)

    def GetGeomType(self, *args):
        r"""
        GetGeomType(self) -> OGRwkbGeometryType

        Return the layer geometry type.

        For more details: :cpp:func:`OGR_L_GetGeomType`

        Returns
        -------
        int
            The geometry type code. The types can be found with
            'osgeo.ogr.wkb' prefix. For example :py:const:`osgeo.ogr.wkbPolygon`.

        """
        return _ogr.Layer_GetGeomType(self, *args)

    def GetGeometryColumn(self, *args):
        r"""
        GetGeometryColumn(self) -> char const *

        This method returns the name of the underlying database column being
        used as the geometry column, or '' if not supported.

        For more details: :cpp:func:`OGR_L_GetGeometryColumn`

        Returns
        -------
        str
            geometry column name.

        """
        return _ogr.Layer_GetGeometryColumn(self, *args)

    def GetFIDColumn(self, *args):
        r"""
        GetFIDColumn(self) -> char const *

        This method returns the name of the underlying database column being
        used as the FID column, or '' if not supported.

        For more details: :cpp:func:`OGR_L_GetFIDColumn`

        Returns
        -------
        str
            fid column name.

        """
        return _ogr.Layer_GetFIDColumn(self, *args)

    def GetFeature(self, *args):
        r"""
        GetFeature(self, fid) -> Feature

        Fetch a feature by its identifier.

        For more details: :cpp:func:`OGR_L_GetFeature`

        Use :py:func:`TestCapability` with (:py:const:`osgeo.ogr.OLCRandomRead`)
        to establish if this layer supports efficient random access reading via
        :py:func:`GetFeature`; However, the call should always work if the feature exists.

        Sequential reads (with :py:func:`GetNextFeature`) are generally
        considered interrupted by a :py:func:`GetFeature` call.

        Parameters
        ----------
        fid : int
            The feature id of the feature to read.

        Returns
        -------
        Feature
            A new feature now owned by the caller, or None on failure.
            The returned feature should be deleted with :py:func:`Destroy`.

        """
        return _ogr.Layer_GetFeature(self, *args)

    def GetNextFeature(self, *args):
        r"""
        GetNextFeature(self) -> Feature

        Fetch the next available feature from this layer.

        For more details: :cpp:func:`OGR_L_GetNextFeature`

        Returns
        -------
        Feature
            A feature or None if no more features are available.

        """
        return _ogr.Layer_GetNextFeature(self, *args)

    def SetNextByIndex(self, *args):
        r"""
        SetNextByIndex(self, new_index) -> OGRErr

        Move read cursor to the nIndex'th feature in the current resultset.

        For more details: :cpp:func:`OGR_L_SetNextByIndex`

        Parameters
        ----------
        new_index : int
            The index indicating how many steps into the result set to seek.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success or an error code.

        """
        return _ogr.Layer_SetNextByIndex(self, *args)

    def SetFeature(self, *args):
        r"""
        SetFeature(self, feature) -> OGRErr

        Rewrite an existing feature.

        For more details: :cpp:func:`OGR_L_SetFeature`

        To set a feature, but create it if it doesn't exist see :py:meth:`.Layer.UpsertFeature`.

        Parameters
        ----------
        feature : Feature
            The feature to write.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` if the operation works,
            otherwise an appropriate error code
            (e.g :py:const:`osgeo.ogr.OGRERR_NON_EXISTING_FEATURE` if the
            feature does not exist).

        """
        return _ogr.Layer_SetFeature(self, *args)

    def CreateFeature(self, *args):
        r"""
        CreateFeature(self, feature) -> OGRErr

        Create and write a new feature within a layer.

        For more details: :cpp:func:`OGR_L_CreateFeature`

        To create a feature, but set it if it exists see :py:meth:`.Layer.UpsertFeature`.

        Parameters
        ----------
        feature : Feature
            The feature to write to disk.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_CreateFeature(self, *args)

    def UpsertFeature(self, *args):
        r"""
        UpsertFeature(self, feature) -> OGRErr

        Rewrite an existing feature or create a new feature within a layer.

        For more details: :cpp:func:`OGR_L_UpsertFeature`

        Parameters
        ----------
        feature : Feature
            The feature to write to disk.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_UpsertFeature(self, *args)

    def UpdateFeature(self, *args):
        r"""UpdateFeature(self, feature, nUpdatedFieldsCount, nUpdatedGeomFieldsCount, bUpdateStyleString) -> OGRErr"""
        return _ogr.Layer_UpdateFeature(self, *args)

    def DeleteFeature(self, *args):
        r"""
        DeleteFeature(self, fid) -> OGRErr

        Delete feature from layer.

        For more details: :cpp:func:`OGR_L_DeleteFeature`

        Parameters
        ----------
        fid : int
            The feature id to be deleted from the layer

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` if the operation works,
            otherwise an appropriate error code
            (e.g :py:const:`osgeo.ogr.OGRERR_NON_EXISTING_FEATURE`)
            if the feature does not exist.

        """
        return _ogr.Layer_DeleteFeature(self, *args)

    def SyncToDisk(self, *args):
        r"""
        SyncToDisk(self) -> OGRErr

        Flush pending changes to disk.

        For more details: :cpp:func:`OGR_L_SyncToDisk`

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` if no error occurs
            (even if nothing is done) or an error code.

        """
        return _ogr.Layer_SyncToDisk(self, *args)

    def GetLayerDefn(self, *args):
        r"""
        GetLayerDefn(self) -> FeatureDefn

        Fetch the schema information for this layer.

        For more details: :cpp:func:`OGR_L_GetLayerDefn`

        Returns
        -------
        FeatureDefn
            The feature definition.

        """
        return _ogr.Layer_GetLayerDefn(self, *args)

    def GetFeatureCount(self, *args, **kwargs):
        r"""
        GetFeatureCount(self, force=1) -> GIntBig

        Fetch the feature count in this layer.

        For more details: :cpp:func:`OGR_L_GetFeatureCount`

        Parameters
        ----------
        force : int
            Flag indicating whether the count should be computed even if
            it is expensive.

        Returns
        -------
        int
            Feature count, -1 if count not known.

        """
        return _ogr.Layer_GetFeatureCount(self, *args, **kwargs)

    def GetExtent(self, *args, **kwargs):
        r"""
        GetExtent(self, force=1, can_return_null=0, geom_field=0)

        Fetch the extent of this layer.

        For more details:

        - :cpp:func:`OGR_L_GetExtent`
        - :cpp:func:`OGR_L_GetExtentEx`

        .. warning:: Check the return order of the bounds.

        Parameters
        ----------
        force : int, default=False
            Flag indicating whether the extent should be computed even if
            it is expensive.
        can_return_null : int, default=False
            Whether None can be returned in the response.
        geom_field : int, default=0
            The index of the geometry field on which to compute the extent.
            Can be iterated over using :py:func:`range` and :py:func:`GetGeomFieldCount`.

        Returns
        -------
        tuple of float
            (minx, maxx, miny, maxy)

        """
        return _ogr.Layer_GetExtent(self, *args, **kwargs)

    def GetExtent3D(self, *args, **kwargs):
        r"""GetExtent3D(self, force=1, can_return_null=0, geom_field=0)"""
        return _ogr.Layer_GetExtent3D(self, *args, **kwargs)

    def TestCapability(self, *args):
        r"""
        TestCapability(self, cap) -> bool

        Test if this layer supported the named capability.

        For more details: :cpp:func:`OGR_L_TestCapability`

        Parameters
        ----------
        cap : str
            The name of the capability to test. These can
            be found in the `osgeo.ogr` namespace. For example,
            :py:const:`osgeo.ogr.OLCRandomRead`.

        Returns
        -------
        int
            True if the layer has the requested capability, or False otherwise.
            Will return False for any unrecognized capabilities.

        """
        return _ogr.Layer_TestCapability(self, *args)

    def CreateField(self, *args, **kwargs):
        r"""
        CreateField(self, field_def, approx_ok=1) -> OGRErr

        Create a new field on a layer.

        For more details: :cpp:func:`OGR_L_CreateField`

        Parameters
        ----------
        field_def : FieldDefn
            The field definition to write to disk.
        approx_ok : bool, default=True
            If True, the field may be created in a slightly different
            form depending on the limitations of the format driver.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_CreateField(self, *args, **kwargs)

    def DeleteField(self, *args):
        r"""
        DeleteField(self, iField) -> OGRErr

        Delete an existing field on a layer.

        For more details: :cpp:func:`OGR_L_DeleteField`

        Parameters
        ----------
        iField : int
            index of the field to delete.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_DeleteField(self, *args)

    def ReorderField(self, *args):
        r"""
        ReorderField(self, iOldFieldPos, iNewFieldPos) -> OGRErr

        Reorder an existing field on a layer.

        For more details: :cpp:func:`OGR_L_ReorderField`

        Parameters
        ----------
        iOldFieldPos : int
            previous position of the field to move. Must be in the
            range [0,GetFieldCount()-1].
        iNewFieldPos : int
            new position of the field to move. Must be in the range
            [0,GetFieldCount()-1].

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_ReorderField(self, *args)

    def ReorderFields(self, *args):
        r"""
        ReorderFields(self, nList) -> OGRErr

        Reorder all the fields of a layer.

        For more details: :cpp:func:`OGR_L_ReorderFields`

        Parameters
        ----------
        nList : list[int]
            A list of GetLayerDefn().GetFieldCount()
            elements which is a permutation of
            [0, GetLayerDefn().GetFieldCount()-1].

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_ReorderFields(self, *args)

    def AlterFieldDefn(self, *args):
        r"""
        AlterFieldDefn(self, iField, field_def, nFlags) -> OGRErr

        Alter the definition of an existing field on a layer.

        For more details: :cpp:func:`OGR_L_AlterFieldDefn`

        Parameters
        ----------
        iField : int
            index of the field whose definition must be altered.
        field_def : FieldDefn
            new field definition
        nFlags : int
            Combination of
            :py:const:`osgeo.ogr.ALTER_NAME_FLAG`,
            :py:const:`osgeo.ogr.ALTER_TYPE_FLAG`,
            :py:const:`osgeo.ogr.ALTER_WIDTH_PRECISION_FLAG`,
            :py:const:`osgeo.ogr.ALTER_NULLABLE_FLAG` and
            :py:const:`osgeo.ogr.ALTER_DEFAULT_FLAG`
            to indicate which of the name and/or type and/or width and precision
            fields and/or nullability from the new field definition must be taken
            into account.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_AlterFieldDefn(self, *args)

    def AlterGeomFieldDefn(self, *args):
        r"""AlterGeomFieldDefn(self, iGeomField, field_def, nFlags) -> OGRErr"""
        return _ogr.Layer_AlterGeomFieldDefn(self, *args)

    def CreateGeomField(self, *args, **kwargs):
        r"""
        CreateGeomField(self, field_def, approx_ok=1) -> OGRErr

        Create a new geometry field on a layer.

        For more details: :cpp:func:`OGR_L_CreateGeomField`

        Parameters
        ----------
        field_def : GeomFieldDefn
            The geometry field definition to write to disk.
        approx_ok : bool, default=True
            If True, the field may be created in a slightly different
            form depending on the limitations of the format driver.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_CreateGeomField(self, *args, **kwargs)

    def StartTransaction(self, *args):
        r"""
        StartTransaction(self) -> OGRErr

        For datasources which support transactions, this creates a transaction.

        For more details: :cpp:func:`OGR_L_StartTransaction`

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_StartTransaction(self, *args)

    def CommitTransaction(self, *args):
        r"""
        CommitTransaction(self) -> OGRErr

        For datasources which support transactions, this commits a transaction.

        For more details: :cpp:func:`OGR_L_CommitTransaction`

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_CommitTransaction(self, *args)

    def RollbackTransaction(self, *args):
        r"""
        RollbackTransaction(self) -> OGRErr

        Roll back a datasource to its state before the start of the current transaction.

        For more details: :cpp:func:`OGR_L_RollbackTransaction`

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success.

        """
        return _ogr.Layer_RollbackTransaction(self, *args)

    def FindFieldIndex(self, *args):
        r"""
        FindFieldIndex(self, pszFieldName, bExactMatch) -> int

        Find the index of field in a layer.

        For more details: :cpp:func:`OGR_L_FindFieldIndex`

        Returns
        -------
        int
            field index, or -1 if the field doesn't exist

        """
        return _ogr.Layer_FindFieldIndex(self, *args)

    def GetSpatialRef(self, *args):
        r"""
        GetSpatialRef(self) -> SpatialReference

        Fetch the spatial reference system for this layer.

        For more details: :cpp:func:`OGR_L_GetSpatialRef`

        Returns
        -------
        SpatialReference
            spatial reference, or ``None`` if there isn't one.

        """
        return _ogr.Layer_GetSpatialRef(self, *args)

    def GetFeaturesRead(self, *args):
        r"""
        GetFeaturesRead(self) -> GIntBig

        For more details: :cpp:func:`OGR_L_GetFeaturesRead`

        """
        return _ogr.Layer_GetFeaturesRead(self, *args)

    def SetIgnoredFields(self, *args):
        r"""
        SetIgnoredFields(self, options) -> OGRErr

        Set which fields can be omitted when retrieving features from the
        layer.

        For more details: :cpp:func:`OGR_L_SetIgnoredFields`

        Parameters
        ----------
        options : list[str]
            A list of field names.
            If an empty list is passed, the ignored list is cleared.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` if all field names have been resolved
            (even if the driver does not support this method)

        """
        return _ogr.Layer_SetIgnoredFields(self, *args)

    def Intersection(self, *args, **kwargs):
        r"""
        Intersection(self, method_layer, result_layer, options=None, callback=0, callback_data=None) -> OGRErr

        Intersection of two layers.

        For more details: :cpp:func:`OGR_L_Intersection`

        Parameters
        ----------
        method_layer : Layer
            the method layer. Should not be None.
        result_layer : Layer
            the layer where the features resulting from the
            operation are inserted. Should not be None.
        options : list[str], optional
            List of options (empty list is allowed). For example ["PROMOTE_TO_MULTI=YES"].
        callback : callable, optional
            a GDALProgressFunc() compatible callback function for
            reporting progress or ``None``.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        int
            An error code if there was an error or the execution was interrupted,
            :py:const:`osgeo.ogr.OGRERR_NONE` otherwise.

        """
        return _ogr.Layer_Intersection(self, *args, **kwargs)

    def Union(self, *args, **kwargs):
        r"""
        Union(self, method_layer, result_layer, options=None, callback=0, callback_data=None) -> OGRErr

        Union of two layers.

        For more details: :cpp:func:`OGR_L_Union`

        The first geometry field is always used.

        Parameters
        ----------
        method_layer : Layer
            the method layer. Should not be None.
        result_layer : Layer
            the layer where the features resulting from the
            operation are inserted. Should not be None.
        options : list[str], optional
            List of options (empty list is allowed). For example ["PROMOTE_TO_MULTI=YES"].
        callback : callable, optional
            a GDALProgressFunc() compatible callback function for
            reporting progress or ``None``.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        int
            An error code if there was an error or the execution was interrupted,
            :py:const:`osgeo.ogr.OGRERR_NONE` otherwise.

        """
        return _ogr.Layer_Union(self, *args, **kwargs)

    def SymDifference(self, *args, **kwargs):
        r"""
        SymDifference(self, method_layer, result_layer, options=None, callback=0, callback_data=None) -> OGRErr

        Symmetrical difference of two layers.

        For more details: :cpp:func:`OGR_L_SymDifference`

        Parameters
        ----------
        method_layer : Layer
            the method layer. Should not be None.
        result_layer : Layer
            the layer where the features resulting from the
            operation are inserted. Should not be None.
        options : list[str], optional
            List of options (empty list is allowed). For example ["PROMOTE_TO_MULTI=YES"].
        callback : callable, optional
            a GDALProgressFunc() compatible callback function for
            reporting progress or ``None``.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        int
            An error code if there was an error or the execution was interrupted,
            :py:const:`osgeo.ogr.OGRERR_NONE` otherwise.

        """
        return _ogr.Layer_SymDifference(self, *args, **kwargs)

    def Identity(self, *args, **kwargs):
        r"""
        Identity(self, method_layer, result_layer, options=None, callback=0, callback_data=None) -> OGRErr

        Identify the features of this layer with the ones from the identity layer.

        For more details: :cpp:func:`OGR_L_Identity`

        Parameters
        ----------
        method_layer : Layer
            the method layer. Should not be None.
        result_layer : Layer
            the layer where the features resulting from the
            operation are inserted. Should not be None.
        options : list[str], optional
            List of options (empty list is allowed). For example ["PROMOTE_TO_MULTI=YES"].
        callback : callable, optional
            a GDALProgressFunc() compatible callback function for
            reporting progress or ``None``.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        int
            An error code if there was an error or the execution was interrupted,
            :py:const:`osgeo.ogr.OGRERR_NONE` otherwise.

        """
        return _ogr.Layer_Identity(self, *args, **kwargs)

    def Update(self, *args, **kwargs):
        r"""
        Update(self, method_layer, result_layer, options=None, callback=0, callback_data=None) -> OGRErr

        Update this layer with features from the update layer.

        For more details: :cpp:func:`OGR_L_Update`

        Parameters
        ----------
        method_layer : Layer
            the method layer. Should not be None.
        result_layer : Layer
            the layer where the features resulting from the
            operation are inserted. Should not be None.
        options : list[str], optional
            List of options (empty list is allowed). For example ["PROMOTE_TO_MULTI=YES"].
        callback : Callable, optional
            a GDALProgressFunc() compatible callback function for
            reporting progress or None.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        int
            An error code if there was an error or the execution was interrupted,
            :py:const:`osgeo.ogr.OGRERR_NONE` otherwise.

        """
        return _ogr.Layer_Update(self, *args, **kwargs)

    def Clip(self, *args, **kwargs):
        r"""
        Clip(self, method_layer, result_layer, options=None, callback=0, callback_data=None) -> OGRErr

        Clip off areas that are not covered by the method layer.

        For more details: :cpp:func:`OGR_L_Clip`

        Parameters
        ----------
        method_layer : Layer
            the method layer. Should not be None.
        result_layer : Layer
            the layer where the features resulting from the
            operation are inserted. Should not be None.
        options : list[str], optional
            List of options (empty list is allowed). For example ["PROMOTE_TO_MULTI=YES"].
        callback : callable, optional
            a GDALProgressFunc() compatible callback function for
            reporting progress or ``None``.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        int
            An error code if there was an error or the execution was interrupted,
            :py:const:`osgeo.ogr.OGRERR_NONE` otherwise.

        """
        return _ogr.Layer_Clip(self, *args, **kwargs)

    def Erase(self, *args, **kwargs):
        r"""
        Erase(self, method_layer, result_layer, options=None, callback=0, callback_data=None) -> OGRErr

        Remove areas that are covered by the method layer.

        For more details: :cpp:func:`OGR_L_Erase`

        Parameters
        ----------
        method_layer : Layer
            the method layer. Should not be None.
        result_layer : Layer
            the layer where the features resulting from the
            operation are inserted. Should not be None.
        options : list[str], optional
            List of options (empty list is allowed). For example ["PROMOTE_TO_MULTI=YES"].
        callback : callable, optional
            a GDALProgressFunc() compatible callback function for
            reporting progress or ``None``.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        int
            An error code if there was an error or the execution was interrupted,
            :py:const:`osgeo.ogr.OGRERR_NONE` otherwise.

        """
        return _ogr.Layer_Erase(self, *args, **kwargs)

    def GetStyleTable(self, *args):
        r"""
        GetStyleTable(self) -> StyleTable

        Get style table.

        For more details: :cpp:func:`OGR_L_GetStyleTable`

        """
        return _ogr.Layer_GetStyleTable(self, *args)

    def SetStyleTable(self, *args):
        r"""
        SetStyleTable(self, table)

        Set style table.

        For more details: :cpp:func:`OGR_L_SetStyleTable`

        """
        return _ogr.Layer_SetStyleTable(self, *args)

    def ExportArrowArrayStreamPyCapsule(self, *args):
        r"""ExportArrowArrayStreamPyCapsule(self, options=None) -> PyObject *"""
        return _ogr.Layer_ExportArrowArrayStreamPyCapsule(self, *args)

    def GetArrowStream(self, *args):
        r"""GetArrowStream(self, options=None) -> ArrowArrayStream"""
        return _ogr.Layer_GetArrowStream(self, *args)

    def IsArrowSchemaSupported(self, *args):
        r"""IsArrowSchemaSupported(self, schema, options=None)"""
        return _ogr.Layer_IsArrowSchemaSupported(self, *args)

    def CreateFieldFromArrowSchema(self, *args):
        r"""CreateFieldFromArrowSchema(self, schema, options=None) -> OGRErr"""
        return _ogr.Layer_CreateFieldFromArrowSchema(self, *args)

    def WriteArrowBatch(self, *args):
        r"""WriteArrowBatch(self, schema, array, options=None) -> OGRErr"""
        return _ogr.Layer_WriteArrowBatch(self, *args)

    def WriteArrowStreamCapsule(self, *args):
        r"""WriteArrowStreamCapsule(self, capsule, createFieldsFromSchema, options=None) -> OGRErr"""
        return _ogr.Layer_WriteArrowStreamCapsule(self, *args)

    def WriteArrowSchemaAndArrowArrayCapsule(self, *args):
        r"""WriteArrowSchemaAndArrowArrayCapsule(self, schemaCapsule, arrayCapsule, createFieldsFromSchema, options=None) -> OGRErr"""
        return _ogr.Layer_WriteArrowSchemaAndArrowArrayCapsule(self, *args)

    def GetGeometryTypes(self, *args, **kwargs):
        r"""
        GetGeometryTypes(self, geom_field=0, flags=0, callback=0, callback_data=None)

        Get actual geometry types found in features.

        For more details: :cpp:func:`OGR_L_GetGeometryTypes`

        Parameters
        ----------
        geom_field : int, optional
            index of the geometry field
        flags : int, optional
            0, or a combination of :py:const:`osgeo.ogr.GGT_COUNT_NOT_NEEDED`,
            :py:const:`osgeo.ogr.GGT_STOP_IF_MIXED` and
            :py:const:`osgeo.ogr.GGT_GEOMCOLLECTIONZ_TINZ`
        callback : callable, optional
            a GDALProgressFunc() compatible callback function for
            cancellation or None.
        callback_data : any
            Argument to be passed to 'callback'. May be None.

        Returns
        -------
        dict
            A dictionary whose keys are :py:const:`osgeo.ogr.wkbXXXX` constants and
            values the corresponding number of geometries of that type in the layer.

        """
        return _ogr.Layer_GetGeometryTypes(self, *args, **kwargs)

    def GetSupportedSRSList(self, *args, **kwargs):
        r"""GetSupportedSRSList(self, geom_field=0)"""
        return _ogr.Layer_GetSupportedSRSList(self, *args, **kwargs)

    def SetActiveSRS(self, *args):
        r"""SetActiveSRS(self, geom_field, srs) -> OGRErr"""
        return _ogr.Layer_SetActiveSRS(self, *args)

    def Reference(self):
      "For backwards compatibility only."
      pass

    def Dereference(self):
      "For backwards compatibility only."
      pass

    def __len__(self):
        """Returns the number of features in the layer"""
        return self.GetFeatureCount()

    # To avoid __len__ being called when testing boolean value
    # which can have side effects (#4758)
    def __nonzero__(self):
        return True

    # For Python 3 compat
    __bool__ = __nonzero__

    def __getitem__(self, value):
        """Support list and slice -like access to the layer.
        layer[0] would return the first feature on the layer.
        layer[0:4] would return a list of the first four features."""
        if isinstance(value, slice):
            import sys
            output = []
            if value.stop == sys.maxsize:
    #for an unending slice, sys.maxsize is used
    #We need to stop before that or GDAL will write an
    ##error to stdout
                stop = len(self) - 1
            else:
                stop = value.stop
            for i in range(value.start, stop, value.step):
                feature = self.GetFeature(i)
                if feature:
                    output.append(feature)
                else:
                    return output
            return output
        if isinstance(value, int):
            if value > len(self) - 1:
                raise IndexError
            return self.GetFeature(value)
        else:
            raise TypeError("Input %s is not of IntType or SliceType" % type(value))

    def CreateFields(self, fields):
        """Create a list of fields on the Layer"""
        for i in fields:
            self.CreateField(i)

    def __enter__(self):
        """Method called when using Dataset.ExecuteSQL() as a context manager"""
        if hasattr(self, "_dataset_weak_ref"):
            self._dataset_strong_ref = self._dataset_weak_ref()
            assert self._dataset_strong_ref is not None
            del self._dataset_weak_ref
            return self
        raise Exception("__enter__() called in unexpected situation")

    def __exit__(self, *args):
        """Method called when using Dataset.ExecuteSQL() as a context manager"""
        if hasattr(self, "_dataset_strong_ref"):
            self._dataset_strong_ref.ReleaseResultSet(self)
            del self._dataset_strong_ref

    def __iter__(self):
        self.ResetReading()
        while True:
            feature = self.GetNextFeature()
            if not feature:
                break
            yield feature

    def schema(self):
        output = []
        defn = self.GetLayerDefn()
        for n in range(defn.GetFieldCount()):
            output.append(defn.GetFieldDefn(n))
        return output
    schema = property(schema)


    def __arrow_c_stream__(self, requested_schema=None):
        """
        Export to a C ArrowArrayStream PyCapsule, according to
        https://arrow.apache.org/docs/format/CDataInterface/PyCapsuleInterface.html

        Also note that only one active stream can be queried at a time for a
        given layer.

        To specify options how the ArrowStream should be generated, use
        the GetArrowArrayStreamInterface(self, options) method

        Parameters
        ----------
        requested_schema : object, optional
            The schema to which the stream should be cast. Passed as a PyCapsule
            containing a C ``ArrowSchema`` representation of the requested schema.
            Currently, this is not supported and will raise a ``NotImplementedError``
            if the schema is not ``None``. Defaults to ``None``.

        Returns
        -------
        object
            A PyCapsule containing a C ``ArrowArrayStream`` struct.
        """

        if requested_schema is not None:
            raise NotImplementedError("requested_schema != None not implemented")

        return self.ExportArrowArrayStreamPyCapsule()


    def GetArrowArrayStreamInterface(self, options = []):
        """
        Return a proxy object that implements the __arrow_c_stream__() method,
        but allows the user to pass options.

        Parameters
        ----------
        options : list of str or dict, optional
            Options such as ``INCLUDE_FID=NO``, ``MAX_FEATURES_IN_BATCH=<number>``, etc.

        Returns
        -------
        object
            A proxy object that implements the ``__arrow_c_stream__()`` method.
        """

        class ArrowArrayStreamInterface:
            def __init__(self, lyr, options):
                self.lyr = lyr
                self.options = options

            def __arrow_c_stream__(self, requested_schema=None):
                """
                Export to a C ArrowArrayStream PyCapsule, according to
                https://arrow.apache.org/docs/format/CDataInterface/PyCapsuleInterface.html

                Also note that only one active stream can be queried at a time for a
                given layer.

                To specify options how the ArrowStream should be generated, use
                the GetArrowArrayStreamInterface(self, options) method

                Parameters
                ----------
               requested_schema : object, optional
                   The schema to which the stream should be cast. Passed as a PyCapsule
                   containing a C ``ArrowSchema`` representation of the requested schema.
                   Currently, this is not supported and will raise a ``NotImplementedError``
                   if the schema is not ``None``. Defaults to ``None``.

                Returns
                -------
                object
                    A PyCapsule containing a C ``ArrowArrayStream`` struct.
                """
                if requested_schema is not None:
                    raise NotImplementedError("requested_schema != None not implemented")

                return self.lyr.ExportArrowArrayStreamPyCapsule(self.options)

        return ArrowArrayStreamInterface(self, options)


    def GetArrowStreamAsPyArrow(self, options = []):
        """ Return an ArrowStream as PyArrow Schema and Array objects """

        import pyarrow as pa

        class Stream:
            def __init__(self, stream):
                self.stream = stream
                self.end_of_stream = False

            def schema(self):
                """ Return the schema as a PyArrow DataType """

                schema = self.stream.GetSchema()
                if schema is None:
                    raise Exception("cannot get schema")
                return pa.DataType._import_from_c(schema._getPtr())

            schema = property(schema)

            def __enter__(self):
                return self

            def __exit__(self, type, value, tb):
                self.end_of_stream = True
                self.stream = None

            def GetNextRecordBatch(self):
                """ Return the next RecordBatch as a PyArrow StructArray, or None at end of iteration """

                array = self.stream.GetNextRecordBatch()
                if array is None:
                    return None
                return pa.Array._import_from_c(array._getPtr(), self.schema)

            def __iter__(self):
                """ Return an iterator over record batches as a PyArrow StructArray """
                if self.end_of_stream:
                    raise Exception("Stream has already been iterated over")

                while True:
                    batch = self.GetNextRecordBatch()
                    if not batch:
                        break
                    yield batch
                self.end_of_stream = True
                self.stream = None

        stream = self.GetArrowStream(options)
        if not stream:
            raise Exception("GetArrowStream() failed")
        return Stream(stream)


    def GetArrowStreamAsNumPy(self, options = []):
        """ Return an ArrowStream as NumPy Array objects.
            A specific option to this method is USE_MASKED_ARRAYS=YES/NO (default is YES).
        """

        from osgeo import gdal_array

        class Stream:
            def __init__(self, stream, use_masked_arrays):
                self.stream = stream
                self.schema = stream.GetSchema()
                self.end_of_stream = False
                self.use_masked_arrays = use_masked_arrays

            def __enter__(self):
                return self

            def __exit__(self, type, value, tb):
                self.end_of_stream = True
                self.schema = None
                self.stream = None

            def GetNextRecordBatch(self):
                """ Return the next RecordBatch as a dictionary of Numpy arrays, or None at end of iteration """

                array = self.stream.GetNextRecordBatch()
                if array is None:
                    return None

                ret = gdal_array._RecordBatchAsNumpy(array._getPtr(),
                                                     self.schema._getPtr(),
                                                     array)
                if ret is None:
                    gdal_array._RaiseException()
                    return ret
                for key, val in ret.items():
                    if isinstance(val, dict):
                        if self.use_masked_arrays:
                            import numpy.ma as ma
                            ret[key] = ma.masked_array(val["data"], val["mask"])
                        else:
                            ret[key] = val["data"]
                return ret

            def __iter__(self):
                """ Return an iterator over record batches as a dictionary of Numpy arrays """

                if self.end_of_stream:
                    raise Exception("Stream has already been iterated over")

                try:
                    while True:
                        batch = self.GetNextRecordBatch()
                        if not batch:
                            break
                        yield batch
                finally:
                    self.end_of_stream = True
                    self.stream = None

        stream = self.GetArrowStream(options)
        if not stream:
            raise Exception("GetArrowStream() failed")

        use_masked_arrays = True
        for opt in options:
            opt = opt.upper()
            if opt.startswith('USE_MASKED_ARRAYS='):
                use_masked_arrays = opt[len('USE_MASKED_ARRAYS='):] in ('YES', 'TRUE', 'ON', '1')

        return Stream(stream, use_masked_arrays)


    def IsPyArrowSchemaSupported(self, pa_schema, options=[]):
        """Returns whether the passed pyarrow Schema is supported by the layer, as a tuple (success: bool, errorMsg: str).

           This may be used as a preliminary check before calling WritePyArrowBatch()
        """

        import pyarrow as pa
        schema = ArrowSchema()
        pa_schema._export_to_c(schema._getPtr())
        return self.IsArrowSchemaSupported(schema, options)


    def CreateFieldFromPyArrowSchema(self, pa_schema, options=[]):
        """Create a field from the passed pyarrow Schema."""

        import pyarrow as pa
        schema = ArrowSchema()
        pa_schema._export_to_c(schema._getPtr())
        return self.CreateFieldFromArrowSchema(schema, options)


    def WriteArrow(self, obj, requested_schema=None, createFieldsFromSchema=None, options=[]):
        """Write the content of the passed object, which must implement the
           __arrow_c_stream__ or __arrow_c_array__ interface, into the layer.

           Parameters
           ----------
           obj : object
               Object implementing the __arrow_c_stream__ or __arrow_c_array__ interface

           requested_schema : object, optional
               The schema to which the stream should be casted. Passed as a PyCapsule
               containing a C ArrowSchema representation of the requested schema, or
               as an object implementing the ``__arrow_c_schema__`` interface.
               Defaults to ``None``.

           createFieldsFromSchema : bool or None, optional
               Whether :py:meth:`Layer.CreateFieldFromArrowSchema` should be called.
               If ``None`` is specified, it is called if no fields have been created yet.
               Defaults to ``None``.

           options : list[str]
               Options to pass to OGRLayer::CreateFieldFromArrowSchema() and OGRLayer::WriteArrowBatch()

        """

        if createFieldsFromSchema is None:
            createFieldsFromSchema = -1
        elif createFieldsFromSchema is True:
            createFieldsFromSchema = 1
        else:
            createFieldsFromSchema = 0

        if requested_schema is not None and hasattr(requested_schema, "__arrow_c_schema__"):
            requested_schema = requested_schema.__arrow_c_schema__()

        if hasattr(obj, "__arrow_c_stream__"):
            if requested_schema:
                stream_capsule = obj.__arrow_c_stream__(requested_schema=requested_schema)
            else:
    # DuckDB 1.1 doesn't support the requested_schema argument at all
                stream_capsule = obj.__arrow_c_stream__()

            return self.WriteArrowStreamCapsule(stream_capsule, createFieldsFromSchema, options)

        if hasattr(obj, "__arrow_c_array__"):
            schema_capsule, array_capsule = obj.__arrow_c_array__(requested_schema=requested_schema)
            return self.WriteArrowSchemaAndArrowArrayCapsule(schema_capsule, array_capsule, createFieldsFromSchema, options)

        raise Exception("Passed object does not implement the __arrow_c_stream__ or __arrow_c_array__ interface.")


    def WritePyArrow(self, pa_batch, options=[]):
        """Write the content of the passed PyArrow batch (either a pyarrow.Table, a pyarrow.RecordBatch or a pyarrow.StructArray) into the layer.

           See also the WriteArrow() method to be independent of PyArrow
        """

        import pyarrow as pa

    # Is it a pyarrow.Table ?
        if hasattr(pa_batch, "to_batches"):
            for batch in pa_batch.to_batches():
                if self.WritePyArrow(batch, options=options) != OGRERR_NONE:
                    return OGRERR_FAILURE
            return OGRERR_NONE

    # Is it a pyarrow.RecordBatch ?
        if hasattr(pa_batch, "columns") and hasattr(pa_batch, "schema"):
            array = pa.StructArray.from_arrays(pa_batch.columns, names=pa_batch.schema.names)
            return self.WritePyArrow(array, options=options)

    # Assume it is a pyarrow.StructArray
        schema = ArrowSchema()
        array = ArrowArray()
        pa_batch._export_to_c(array._getPtr(), schema._getPtr())
        return self.WriteArrowBatch(schema, array, options)



# Register Layer in _ogr:
_ogr.Layer_swigregister(Layer)
class Feature(object):
    r"""

    Python proxy of an :cpp:class:`OGRFeature`.

    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_Feature

    def __init__(self, *args, **kwargs):
        r"""
        __init__(self, feature_def) -> Feature


        Parameters
        ----------
        feature_def : FeatureDefn
            The feature definition to which the feature will adhere.

        """
        _ogr.Feature_swiginit(self, _ogr.new_Feature(*args, **kwargs))

    def GetDefnRef(self, *args):
        r"""
        GetDefnRef(self) -> FeatureDefn


        Fetch the :py:class:`FeatureDefn` associated with this Feature.

        See :cpp:func:`OGRFeature::GetDefnRef()`.

        Returns
        -------
        FeatureDefn

        """
        return _ogr.Feature_GetDefnRef(self, *args)

    def SetGeometry(self, *args):
        r"""
        SetGeometry(self, geom) -> OGRErr


        Set feature geometry.

        This function updates the features geometry, and operates exactly as
        :py:meth:`SetGeometryDirectly`, except that this function does not assume
        ownership of the passed geometry, but instead makes a copy of it.

        See :cpp:func:`OGRFeature::SetGeometry`.

        This method has only an effect on the in-memory feature object. If
        this object comes from a layer and the modifications must be
        serialized back to the datasource, :py:meth:`Layer.SetFeature` must be used
        afterwards. Or if this is a new feature, :py:meth:`Layer.CreateFeature` must be
        used afterwards.

        Parameters
        ----------
        geom : Geometry
            new geometry to apply to feature.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` if successful, or
            :py:const:`OGR_UNSUPPORTED_GEOMETRY_TYPE` if the geometry type is illegal for
            the :py:class:`FeatureDefn` (checking not yet implemented).

        """
        return _ogr.Feature_SetGeometry(self, *args)

    def SetGeometryDirectly(self, geom):
        """
        Set feature geometry.

        This function updates the features geometry, and operates exactly as
        :py:meth:`SetGeometry`, except that this function assumes ownership of the
        passed geometry (even in case of failure of that function).

        See :cpp:func:`OGRFeature::SetGeometryDirectly`.

        This method has only an effect on the in-memory feature object. If
        this object comes from a layer and the modifications must be
        serialized back to the datasource, :py:meth:`Layer.SetFeature` must be used
        afterwards. Or if this is a new feature, :py:meth:`Layer.CreateFeature` must be
        used afterwards.

        Parameters
        ----------
        geom : Geometry
            geometry to apply to feature.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` if successful, or
            :py:const:`OGR_UNSUPPORTED_GEOMETRY_TYPE` if the geometry type is illegal for
            the :py:class:`FeatureDefn` (checking not yet implemented).
        """
        ret = _ogr.Feature_SetGeometryDirectly(self, geom)
        if ret == OGRERR_NONE:
            self._add_geom_ref(geom)
        return ret



    def GetGeometryRef(self, *args):
        r"""
        GetGeometryRef(self) -> Geometry

        Return the feature geometry

        The lifetime of the returned geometry is bound to the one of its belonging
        feature.

        See :cpp:func:`OGRFeature::GetGeometryRef`

        The :py:func:`Feature.geometry` method is also available as an alias of :py:func:`Feature.GetGeometryRef`.

        Returns
        -------
        Geometry
            the geometry, or None.

        """
        val = _ogr.Feature_GetGeometryRef(self, *args)

        self._add_geom_ref(val)


        return val


    def SetGeomField(self, *args):
        r"""
        SetGeomField(self, iField, geom) -> OGRErr
        SetGeomField(self, field_name, geom) -> OGRErr


        Set feature geometry of a specified geometry field.

        This function updates the features geometry, and operates exactly as
        :py:meth:`SetGeomFieldDirectly`, except that this function does not assume
        ownership of the passed geometry, but instead makes a copy of it.

        See :cpp:func:`OGRFeature::SetGeomField`.

        Parameters
        ----------
        fld_index : int or str
            Geometry field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.
        geom : Geometry
            handle to the new geometry to apply to feature.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` if successful, or
            :py:const:`OGR_UNSUPPORTED_GEOMETRY_TYPE` if the geometry type is illegal for
            the :py:class:`FeatureDefn` (checking not yet implemented).

        """
        return _ogr.Feature_SetGeomField(self, *args)

    def SetGeomFieldDirectly(self, field, geom):
        """
        Set feature geometry of a specified geometry field.

        This function updates the features geometry, and operates exactly as
        :py:meth:`SetGeomField`, except that this function assumes ownership of the
        passed geometry (even in case of failure of that function).

        See :cpp:func:`OGRFeature::SetGeomFieldDirectly`.

        Parameters
        ----------
        fld_index : int or str
            Geometry field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.
        geom : Geometry
            handle to the new geometry to apply to feature.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` if successful, or
            :py:const:`OGR_UNSUPPORTED_GEOMETRY_TYPE` if the geometry type is illegal for
            the :py:class:`FeatureDefn` (checking not yet implemented).
        """
        ret = _ogr.Feature_SetGeomFieldDirectly(self, field, geom)
        if ret == OGRERR_NONE:
            self._add_geom_ref(geom)
        return ret



    def GetGeomFieldRef(self, *args):
        r"""
        GetGeomFieldRef(self, iField) -> Geometry
        GetGeomFieldRef(self, field_name) -> Geometry


        Fetch a feature :py:class:`Geometry`.

        See :cpp:func:`OGRFeature::GetGeomFieldRef`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        Geometry


        """
        val = _ogr.Feature_GetGeomFieldRef(self, *args)

        self._add_geom_ref(val)


        return val


    def Clone(self, *args):
        r"""
        Clone(self) -> Feature

        Duplicate a Feature.
        See :cpp:func:`OGRFeature::Clone`.

        Returns
        -------
        Feature

        """
        return _ogr.Feature_Clone(self, *args)

    def Equal(self, *args):
        r"""
        Equal(self, feature) -> bool


        Test if two features are the same.

        Two features are considered equal if they reference the
        same :py:class:`FeatureDefn`, have the same field values, and the same geometry
        (as tested by :py:func:`Geometry.Equal`) as well as the same feature id.

        See :cpp:func:`OGRFeature::Equal`.

        Parameters
        ----------
        feature : Feature
            feature to test this one against

        Returns
        -------
        bool

        """
        return _ogr.Feature_Equal(self, *args)

    def GetFieldCount(self, *args):
        r"""
        GetFieldCount(self) -> int


        Fetch number of fields on this feature This will always be the same as
        the field count for the :py:class:`FeatureDefn`.

        See :cpp:func:`OGRFeature::GetFieldCount`.

        Returns
        -------
        int
            count of fields.

        """
        return _ogr.Feature_GetFieldCount(self, *args)

    def GetFieldDefnRef(self, *args):
        r"""
        GetFieldDefnRef(self, id) -> FieldDefn
        GetFieldDefnRef(self, field_name) -> FieldDefn


        Fetch definition for this field.

        See :cpp:func:`OGRFeature::GetFieldDefnRef`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        FieldDefn
            a reference to the field definition. This reference should
            not be modified.

        """
        return _ogr.Feature_GetFieldDefnRef(self, *args)

    def GetGeomFieldCount(self, *args):
        r"""
        GetGeomFieldCount(self) -> int


        Fetch number of geometry fields on this feature This will always be
        the same as the geometry field count for the :py:class:`FeatureDefn`.

        See :cpp:func:`OGRFeature::GetGeomFieldCount`.

        Returns
        -------
        int
            count of geometry fields.

        """
        return _ogr.Feature_GetGeomFieldCount(self, *args)

    def GetGeomFieldDefnRef(self, *args):
        r"""
        GetGeomFieldDefnRef(self, id) -> GeomFieldDefn
        GetGeomFieldDefnRef(self, field_name) -> GeomFieldDefn


        Fetch definition for this geometry field.

        See :cpp:func:`OGRFeature::GetGeomFieldDefnRef`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        GeomFieldDefn
            a reference to the field definition.
            Should not be deleted or modified.

        """
        return _ogr.Feature_GetGeomFieldDefnRef(self, *args)

    def GetFieldAsString(self, *args):
        r"""
        GetFieldAsString(self, id) -> char const
        GetFieldAsString(self, field_name) -> char const *


        :py:const:`OFTReal` and :py:const:`OFTInteger` fields will be translated to string using
        sprintf(), but not necessarily using the established formatting rules.
        Other field types, or errors will result in a return value of zero.

        See :cpp:func:`OGRFeature::GetFieldAsString`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        str
            the field value.

        """
        return _ogr.Feature_GetFieldAsString(self, *args)

    def GetFieldAsISO8601DateTime(self, *args):
        r"""
        GetFieldAsISO8601DateTime(self, id, options=None) -> char const
        GetFieldAsISO8601DateTime(self, field_name, options=None) -> char const *


        Fetch :py:const:`OFTDateTime` field value as a ISO8601 representation.

        Return a string like 'YYYY-MM-DDTHH:MM:SS(.sss)?(Z|([+|-]HH:MM))?'
        Milliseconds are omitted if equal to zero.
        Other field types, or errors will result in a return of an empty string.

        See :cpp:func:`OGRFeature::GetFieldAsISO8601DateTime`.

        .. versionadded:: 3.7

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.
        options : dict or str
            Not currently used.

        """
        return _ogr.Feature_GetFieldAsISO8601DateTime(self, *args)

    def GetFieldAsInteger(self, *args):
        r"""
        GetFieldAsInteger(self, id) -> int
        GetFieldAsInteger(self, field_name) -> int


        Fetch field value as a 32-bit integer.

        :py:const:`OFTString` features will be translated using atoi().
        :py:const:`OFTReal` fields will be cast to integer. Other field types, or
        errors will result in a return value of zero.

        See :cpp:func:`OGRFeature::GetFieldAsInteger`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        int
            the field value.

        Examples
        --------
        >>> defn = ogr.FeatureDefn()
        >>> defn.AddFieldDefn(ogr.FieldDefn('my_int', ogr.OFTInteger64))
        >>> feature = ogr.Feature(defn)
        >>> feature['my_int'] = 2**32 + 1
        >>> feature.GetFieldAsInteger('my_int')
        Warning 1: Integer overflow occurred when trying to return 64bit integer. Use GetFieldAsInteger64() instead
        2147483647
        >>> feature.GetFieldAsInteger64('my_int')
        4294967297
        >>> feature.GetField('my_int')
        4294967297

        """
        return _ogr.Feature_GetFieldAsInteger(self, *args)

    def GetFieldAsInteger64(self, *args):
        r"""
        GetFieldAsInteger64(self, id) -> GIntBig
        GetFieldAsInteger64(self, field_name) -> GIntBig


        Fetch field value as integer 64 bit.

        :py:const:`OFTInteger` are promoted to 64 bit. :py:const:`OFTString` features
        will be translated using :cpp:func:`CPLAtoGIntBig`. :py:const:`OFTReal` fields
        will be cast to integer. Other field types, or errors will result in a return
        value of zero.

        See :cpp:func:`OGRFeature::GetFieldAsInteger64`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        int
            the field value.

        """
        return _ogr.Feature_GetFieldAsInteger64(self, *args)

    def GetFieldAsDouble(self, *args):
        r"""
        GetFieldAsDouble(self, id) -> double
        GetFieldAsDouble(self, field_name) -> double

        Fetch field value as a double.

        :py:const:`OFTString` features will be translated using :cpp:func:`CPLAtof`. :py:const:`OFTInteger`
        fields will be cast to double. Other field types, or errors will
        result in a return value of zero.

        See :cpp:func:`OGRFeature::GetFieldAsDouble`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        float
            the field value.

        """
        return _ogr.Feature_GetFieldAsDouble(self, *args)

    def GetFieldAsDateTime(self, *args):
        r"""
        GetFieldAsDateTime(self, id)
        GetFieldAsDateTime(self, field_name)


        Fetch field value as date and time.

        Currently this method only works for :py:const:`OFTDate`, :py:const:`OFTTime`
        and :py:const:`OFTDateTime` fields.

        See :cpp:func:`OGRFeature::GetFieldAsDateTime`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        list
            list containing [ year, month, day, hour, minute, second, timezone flag ]

        Examples
        --------
        >>> from datetime import datetime
        >>> from zoneinfo import ZoneInfo
        >>> defn = ogr.FeatureDefn()
        >>> defn.AddFieldDefn(ogr.FieldDefn('unknown', ogr.OFTDateTime))
        >>> defn.AddFieldDefn(ogr.FieldDefn('local', ogr.OFTDateTime))
        >>> defn.AddFieldDefn(ogr.FieldDefn('utc', ogr.OFTDateTime))
        >>> feature = ogr.Feature(defn)
        >>> feature['unknown'] = datetime.now()
        >>> feature['local'] = datetime.now(ZoneInfo('America/Toronto'))
        >>> feature['utc'] = datetime.now(ZoneInfo('UTC'))
        >>> feature.GetFieldAsDateTime('unknown')
        [2024, 3, 15, 20, 34, 52.594173431396484, 0] # random
        >>> feature.GetFieldAsDateTime('local')
        [2024, 3, 15, 20, 34, 52.59502410888672, 84] # random
        >>> feature.GetFieldAsDateTime('utc')
        [2024, 3, 16, 0, 34, 52.59580993652344, 100] # random

        See Also
        --------
        :py:func:`Feature.GetFieldAsISO8601DateTime`

        """
        return _ogr.Feature_GetFieldAsDateTime(self, *args)

    def GetFieldAsIntegerList(self, *args):
        r"""
        GetFieldAsIntegerList(self, id)
        GetFieldAsIntegerList(self, field_name)


        Fetch field value as a list of integers.

        Currently this function only works for :py:const:`OFTIntegerList` fields.

        This function is the same as the C++ method
        :cpp:func:`OGRFeature::GetFieldAsIntegerList`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        list
            the field value.

        """
        return _ogr.Feature_GetFieldAsIntegerList(self, *args)

    def GetFieldAsInteger64List(self, *args):
        r"""
        GetFieldAsInteger64List(self, id)

        Fetch field value as a list of 64 bit integers.

        Currently this function only works for :py:const:`OFTInteger64List` fields.

        See :cpp:func:`OGRFeature::GetFieldAsInteger64List`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        list
            the field value.

        """
        return _ogr.Feature_GetFieldAsInteger64List(self, *args)

    def GetFieldAsDoubleList(self, *args):
        r"""
        GetFieldAsDoubleList(self, id)
        GetFieldAsDoubleList(self, field_name)


        Fetch field value as a list of doubles.

        Currently this function only works for :py:const:`OFTRealList` fields.

        See :cpp:func:`OGRFeature::GetFieldAsDoubleList`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        list

        Examples
        --------
        >>> defn = ogr.FeatureDefn()
        >>> defn.AddFieldDefn(ogr.FieldDefn('list', ogr.OFTRealList))
        >>> feature = ogr.Feature(defn)
        >>> feature['list'] = [1.1, 2.2, 3.3]
        >>> feature.GetFieldAsDoubleList('list')
        [1.1, 2.2, 3.3]

        """
        return _ogr.Feature_GetFieldAsDoubleList(self, *args)

    def GetFieldAsStringList(self, *args):
        r"""
        GetFieldAsStringList(self, id) -> char **


        Fetch field value as a list of strings.

        Currently this method only works for :py:const:`OFTStringList` fields.

        See :cpp:func:`OGRFeature::GetFieldAsStringList`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        list
            the field value.

        """
        return _ogr.Feature_GetFieldAsStringList(self, *args)

    def GetFieldAsBinary(self, *args):
        r"""
        GetFieldAsBinary(self, id) -> OGRErr
        GetFieldAsBinary(self, field_name) -> OGRErr


        Fetch field value as binary.

        This method only works for :py:const:`OFTBinary` and :py:const:`OFTString` fields.

        See :cpp:func:`OGRFeature::GetFieldAsBinary`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        bytearray

        """
        return _ogr.Feature_GetFieldAsBinary(self, *args)

    def IsFieldSet(self, *args):
        r"""
        IsFieldSet(self, id) -> bool
        IsFieldSet(self, field_name) -> bool


        Test if a field has ever been assigned a value or not.

        See :cpp:func:`OGRFeature::IsFieldSet`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        bool
            ``True`` if the field has been set, otherwise ``False``.

        """
        return _ogr.Feature_IsFieldSet(self, *args)

    def IsFieldNull(self, *args):
        r"""
        IsFieldNull(self, id) -> bool
        IsFieldNull(self, field_name) -> bool


        Test if a field is null.

        See :cpp:func:OGRFeature::`IsFieldNull`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        bool
            ``True`` if the field is null, otherwise ``False``

        """
        return _ogr.Feature_IsFieldNull(self, *args)

    def IsFieldSetAndNotNull(self, *args):
        r"""
        IsFieldSetAndNotNull(self, id) -> bool
        IsFieldSetAndNotNull(self, field_name) -> bool


        Test if a field is set and not null.

        See :cpp:func:`OGRFeature::IsFieldSetAndNotNull`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        bool
            ``True`` if the field is set and not null, otherwise ``False``.

        """
        return _ogr.Feature_IsFieldSetAndNotNull(self, *args)

    def GetFieldIndex(self, *args):
        r"""
        GetFieldIndex(self, field_name) -> int


        Fetch the field index given field name.

        See :cpp:func:`OGRFeature::GetFieldIndex`.

        Parameters
        ----------
        field_name : str
            the name of the field to search for.

        Returns
        -------
        int
            the field index, or -1 if no matching field is found.

        """
        return _ogr.Feature_GetFieldIndex(self, *args)

    def GetGeomFieldIndex(self, *args):
        r"""
        GetGeomFieldIndex(self, field_name) -> int


        Fetch the geometry field index given geometry field name.

        See :cpp:func:`OGRFeature::GetGeomFieldIndex`.

        Parameters
        ----------
        field_name : str
            the name of the geometry field to search for.

        Returns
        -------
        int
            the geometry field index, or -1 if no matching geometry field is found.

        """
        return _ogr.Feature_GetGeomFieldIndex(self, *args)

    def GetFID(self, *args):
        r"""
        GetFID(self) -> GIntBig


        Get feature identifier.
        See :cpp:func:`OGRFeature::GetFID`

        Returns
        -------
        int
            feature id or :py:const:`NullFID` if none has been assigned.

        """
        return _ogr.Feature_GetFID(self, *args)

    def SetFID(self, *args):
        r"""
        SetFID(self, fid) -> OGRErr


        Set the feature identifier.

        For specific types of features this operation may fail on illegal
        features ids. Generally it always succeeds. Feature ids should be
        greater than or equal to zero, with the exception of :py:const:NullFID` (-1)
        indicating that the feature id is unknown.

        See :cpp:func:`OGRFeature::SetFID`.

        Parameters
        ----------
        fid : int
            the new feature identifier value to assign.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` on success, or some other value on failure.

        """
        return _ogr.Feature_SetFID(self, *args)

    def DumpReadable(self, *args):
        r"""
        DumpReadable(self)


        Print this feature in a human readable form.

        This dumps the attributes and geometry. It doesn't include
        definition information other than field types and names nor does it
        report the geometry spatial reference system.

        See :cpp:func:`OGRFeature::DumpReadable`.

        Examples
        --------
        >>> with gdal.OpenEx('poly.shp', gdal.OF_VECTOR) as ds:
        ...     lyr = ds.GetLayer(0)
        ...     feature = lyr.GetNextFeature()
        ...     feature.DumpReadable()
        ...
        OGRFeature(poly):0
          AREA (Real) = 215229.266
          EAS_ID (Integer64) = 168
          PRFEDEA (String) = 35043411
          POLYGON ((479819.84375 4765180.5,479690.1875 4765259.5,479647.0 4765369.5,479730.375 4765400.5,480039.03125 4765539.5,480035.34375 4765558.5,480159.78125 4765610.5,480202.28125 4765482.0,480365.0 4765015.5,480389.6875 4764950.0,480133.96875 4764856.5,480080.28125 4764979.5,480082.96875 4765049.5,480088.8125 4765139.5,480059.90625 4765239.5,480019.71875 4765319.5,479980.21875 4765409.5,479909.875 4765370.0,479859.875 4765270.0,479819.84375 4765180.5)) # no-check

        """
        return _ogr.Feature_DumpReadable(self, *args)

    def DumpReadableAsString(self, *args):
        r"""
        DumpReadableAsString(self, options=None) -> retStringAndCPLFree *


        Return feature information in a human-readable form.
        Returns the text printed by :py:func:`Feature.DumpReadable`.

        Returns
        -------
        str

        """
        return _ogr.Feature_DumpReadableAsString(self, *args)

    def UnsetField(self, *args):
        r"""
        UnsetField(self, id)
        UnsetField(self, field_name)


        Clear a field, marking it as unset.

        See :cpp:func:`OGRFeature::UnsetField`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        """
        return _ogr.Feature_UnsetField(self, *args)

    def SetFieldNull(self, *args):
        r"""
        SetFieldNull(self, id)
        SetFieldNull(self, field_name)


        Clear a field, marking it as null.

        See :cpp:func:`OGRFeature::SetFieldNull`.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.


        """
        return _ogr.Feature_SetFieldNull(self, *args)

    def SetFieldInteger64(self, *args):
        r"""SetFieldInteger64(self, id, value)"""
        return _ogr.Feature_SetFieldInteger64(self, *args)

        # With several override, SWIG cannot dispatch automatically unicode strings
        # to the right implementation, so we have to do it at hand
    def SetField(self, *args) -> "OGRErr":
        """
        SetField(self, fld_index, value: str)
        SetField(self, fld_name, value: str)
        SetField(self, fld_index, value: int)
        SetField(self, fld_name, value: int)
        SetField(self, fld_index, value: float)
        SetField(self, fld_name, value: float)
        SetField(self, fld_index, year: int, month: int, day: int, hour: int, minute: int, second: int|float, tzflag: int)
        SetField(self, fld_name, year: int, month: int, day: int, hour: int, minute: int, second: int|float, tzflag: int)
        SetField(self, fld_index, value: bytes)
        SetField(self, fld_name, value: bytes)
        """

        if len(args) == 2 and args[1] is None:
            return _ogr.Feature_SetFieldNull(self, args[0])

        if len(args) == 2 and isinstance(args[1], int):
            fld_index = args[0]
            if isinstance(fld_index, str):
                fld_index = self._getfieldindex(fld_index)
            return _ogr.Feature_SetFieldInteger64(self, fld_index, args[1])

        if len(args) == 2 and isinstance(args[1], str):
            fld_index = args[0]
            if isinstance(fld_index, str):
                fld_index = self._getfieldindex(fld_index)
            return _ogr.Feature_SetFieldString(self, fld_index, args[1])

        if len(args) == 2 and isinstance(args[1], (bytes, bytearray, memoryview)):
            fld_index = args[0]
            if isinstance(fld_index, str):
                fld_index = self._getfieldindex(fld_index)
            if self.GetFieldType(fld_index) == OFTBinary:
                return self._SetFieldBinary(fld_index, args[1])

        return _ogr.Feature_SetField(self, *args)



    def SetFieldIntegerList(self, *args):
        r"""
        SetFieldIntegerList(self, id, nList)
        void

        Set field to list of integer values.

        This function currently on has an effect of :py:const:`OFTIntegerList`,
        :py:const:`OFTInteger64List`, :py:const:`OFTRealList` fields.

        See :cpp:func:`OGRFeature::SetField`.

        This method has only an effect on the in-memory feature object. If
        this object comes from a layer and the modifications must be
        serialized back to the datasource, :py:meth:`Layer.SetFeature` must be used
        afterwards. Or if this is a new feature, :py:meth:`Layer.CreateFeature` must be
        used afterwards.

        Parameters
        ----------
        id : int
            the field to set, from 0 to :py:meth:`GetFieldCount`-1.
        nList : list
            the values to assign.

        """
        return _ogr.Feature_SetFieldIntegerList(self, *args)

    def SetFieldInteger64List(self, *args):
        r"""
        SetFieldInteger64List(self, id, nList)
        void

        Set field to list of 64 bit integer values.

        This function currently on has an effect of :py:const:`OFTIntegerList`,
        :py:const:`OFTInteger64List`, :py:const:`OFTRealList` fields.

        See :cpp:func:`OGRFeature::SetField`.

        This method has only an effect on the in-memory feature object. If
        this object comes from a layer and the modifications must be
        serialized back to the datasource, :py:meth:`Layer.SetFeature` must be used
        afterwards. Or if this is a new feature, :py:meth:`Layer.CreateFeature` must be
        used afterwards.

        Parameters
        ----------
        id : int
            the field to set, from 0 to :py:meth:`GetFieldCount`-1.
        nList : list
            the values to assign.

        """
        return _ogr.Feature_SetFieldInteger64List(self, *args)

    def SetFieldDoubleList(self, *args):
        r"""
        SetFieldDoubleList(self, id, nList)


        Set field to list of double values.

        This function currently on has an effect of :py:const:`OFTIntegerList`,
        :py:const:`OFTInteger64List`, :py:const:`OFTRealList` fields.

        See :cpp:func:`OGRFeature::SetField`.

        This method has only an effect on the in-memory feature object. If
        this object comes from a layer and the modifications must be
        serialized back to the datasource, :py:meth:`Layer.SetFeature` must be used
        afterwards. Or if this is a new feature, :py:meth:`Layer.CreateFeature` must be
        used afterwards.

        Parameters
        ----------
        id : int
            the field to set, from 0 to :py:meth:`GetFieldCount`-1.
        nList : list
            the values to assign.

        """
        return _ogr.Feature_SetFieldDoubleList(self, *args)

    def SetFieldStringList(self, *args):
        r"""
        SetFieldStringList(self, id, pList)


        Set field to list of strings value.

        This function currently only has an effect of :py:const:`OFTStringList` fields.

        See :cpp:func:`OGRFeature::SetField`.

        This method has only an effect on the in-memory feature object. If
        this object comes from a layer and the modifications must be
        serialized back to the datasource, :py:meth:`Layer.SetFeature` must be used
        afterwards. Or if this is a new feature, :py:meth:`Layer.CreateFeature` must be
        used afterwards.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.
        value : any
            the value to assign.

        """
        return _ogr.Feature_SetFieldStringList(self, *args)

    def _SetFieldBinary(self, *args):
        r"""_SetFieldBinary(self, id, nLen)"""
        return _ogr.Feature__SetFieldBinary(self, *args)

    def SetFieldBinaryFromHexString(self, *args):
        r"""
        SetFieldBinaryFromHexString(self, id, pszValue)
        SetFieldBinaryFromHexString(self, field_name, pszValue)
        """
        return _ogr.Feature_SetFieldBinaryFromHexString(self, *args)

    def SetFrom(self, *args, **kwargs):
        r"""
        SetFrom(self, other, forgiving=1) -> OGRErr

        Set one feature from another.

        Overwrite the contents of this feature from the geometry and
        attributes of another. The other feature does not need to have the
        same :py:class:`FeatureDefn`. Field values are copied by corresponding field
        names. Field types do not have to exactly match. OGR_F_SetField\*()
        function conversion rules will be applied as needed.

        See :cpp:func:`OGRFeature::SetFrom`.

        Parameters
        ----------
        other : Feature
            feature from which geometry and field values will be copied.
        forgiving : bool, default = True
            ``True`` if the operation should continue despite lacking
            output fields matching some of the source fields.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` if the operation succeeds, even if some values are not
            transferred, otherwise an error code.

        """
        return _ogr.Feature_SetFrom(self, *args, **kwargs)

    def SetFromWithMap(self, *args):
        r"""
        SetFromWithMap(self, other, forgiving, nList) -> OGRErr


        Set one feature from another.

        Overwrite the contents of this feature from the geometry and
        attributes of another. The other feature does not need to have the
        same :py:class:`FeatureDefn`. Field values are copied according to the provided
        indices map. Field types do not have to exactly match.
        OGR_F_SetField\*() function conversion rules will be applied as needed.
        This is more efficient than :py:meth:SetFrom` in that this doesn't
        lookup the fields by their names. Particularly useful when the field
        names don't match.

        See :cpp:func:`OGRFeature::SetFrom`.

        Parameters
        ----------
        other : Feature
            handle to the feature from which geometry, and field
            values will be copied.
        forgiving : bool
            ``True`` if the operation should continue despite lacking
            output fields matching some of the source fields.
        nList : list
            Array of the indices of the destination feature's fields
            stored at the corresponding index of the source feature's fields. A
            value of -1 should be used to ignore the source's field. The array
            should not be NULL and be as long as the number of fields in the
            source feature.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` if the operation succeeds, even if some values are not
            transferred, otherwise an error code.

        """
        return _ogr.Feature_SetFromWithMap(self, *args)

    def GetStyleString(self, *args):
        r"""
        GetStyleString(self) -> char const *


        Fetch style string for this feature.

        Set the OGR Feature Style Specification for details on the format of
        this string, and :source_file:`ogr/ogr_featurestyle.h` for services available to parse
        it.

        See :cpp:func:`OGRFeature::GetStyleString`.

        Returns
        -------
        str or None

        """
        return _ogr.Feature_GetStyleString(self, *args)

    def SetStyleString(self, *args):
        r"""
        SetStyleString(self, the_string)


        Set feature style string.

        See :cpp:func:`OGRFeature::SetStyleString`.

        Parameters
        ----------
        the_string : str
            the style string to apply to this feature

        """
        return _ogr.Feature_SetStyleString(self, *args)

    def GetFieldType(self, *args):
        r"""
        GetFieldType(self, id) -> OGRFieldType
        GetFieldType(self, field_name) -> OGRFieldType


        Return the type of the given field.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        int
            field type code (e.g., :py:const:`OFTInteger`)

        """
        return _ogr.Feature_GetFieldType(self, *args)

    def Validate(self, *args):
        r"""
        Validate(self, flags=OGR_F_VAL_ALL, bEmitError=TRUE) -> int


        Validate that a feature meets constraints of its schema.

        The scope of test is specified with the ``flags`` parameter.

        Regarding :py:const:`OGR_F_VAL_WIDTH`, the test is done assuming the string
        width must be interpreted as the number of UTF-8 characters. Some drivers might
        interpret the width as the number of bytes instead. So this test is rather
        conservative (if it fails, then it will fail for all interpretations).

        See :cpp:func:`OGRFeature::Validate`.

        Parameters
        ----------
        flags : int, default = :py:const:`F_VAL_ALL`
            One or more of :py:const:`OGR_F_VAL_NULL`,
            :py:const:`OGR_F_VAL_GEOM_TYPE`, py:const:`OGR_F_VAL_WIDTH` and
            :py:const:`OGR_F_VAL_ALLOW_NULL_WHEN_DEFAULT` combined with
            the with ``|`` operator
        bEmitError : bool, default = True
            TRUE if a CPLError() must be emitted when a check fails

        Returns
        -------
        int
            TRUE if all enabled validation tests pass.

        """
        return _ogr.Feature_Validate(self, *args)

    def FillUnsetWithDefault(self, *args):
        r"""
        FillUnsetWithDefault(self, bNotNullableOnly=FALSE, options=None)


        Fill unset fields with default values that might be defined.

        See :cpp:func:`OGRFeature::FillUnsetWithDefault`.

        Parameters
        ----------
        bNotNullableOnly : bool
            if we should fill only unset fields with a not-null
            constraint.
        options : dict
            unused currently.

        """
        return _ogr.Feature_FillUnsetWithDefault(self, *args)

    def GetNativeData(self, *args):
        r"""
        GetNativeData(self) -> char const *


        Returns the native data for the feature.

        The native data is the representation in a "natural" form that comes
        from the driver that created this feature, or that is aimed at an
        output driver. The native data may be in different format, which is
        indicated by :py:func:`GetNativeMediaType`.

        Note that most drivers do not support storing the native data in the
        feature object, and if they do, generally the ``NATIVE_DATA`` open option
        must be passed at dataset opening.

        The "native data" does not imply it is something more performant or
        powerful than what can be obtained with the rest of the API, but it
        may be useful in round-tripping scenarios where some characteristics
        of the underlying format are not captured otherwise by the OGR
        abstraction.

        See :cpp:func:`OGRFeature::GetNativeData` and :ref:`rfc-60`.

        Returns
        -------
        str
            a string with the native data, or ``None``.

        """
        return _ogr.Feature_GetNativeData(self, *args)

    def GetNativeMediaType(self, *args):
        r"""
        GetNativeMediaType(self) -> char const *


        Returns the native media type for the feature.

        The native media type is the identifier for the format of the native
        data. It follows the IANA RFC 2045
        (see https://en.wikipedia.org/wiki/Media_type), e.g.
        "application/vnd.geo+json" for JSon.

        See :cpp:func:`OGRFeature::GetNativeMediaType` and :ref:`rfc-60`.

        Returns
        -------
        str
            a string with the native media type, or ``None``.

        """
        return _ogr.Feature_GetNativeMediaType(self, *args)

    def SetNativeData(self, *args):
        r"""
        SetNativeData(self, nativeData)


        Sets the native data for the feature.

        The native data is the representation in a "natural" form that comes
        from the driver that created this feature, or that is aimed at an
        output driver. The native data may be in different format, which is
        indicated by :py:meth:`GetNativeMediaType`.

        See :cpp:func:`OGRFeature::SetNativeData` and :ref:`rfc-60`.

        Parameters
        ----------
        nativeData : str
            a string with the native data, or ``None``

        """
        return _ogr.Feature_SetNativeData(self, *args)

    def SetNativeMediaType(self, *args):
        r"""
        SetNativeMediaType(self, nativeMediaType)


        Sets the native media type for the feature.

        The native media type is the identifier for the format of the native
        data. It follows the IANA RFC 2045
        (see https://en.wikipedia.org/wiki/Media_type), e.g.
        "application/vnd.geo+json" for JSon.

        See :cpp:func:`OGRFeature::SetNativeMediaType` and :ref:`rfc-60`.

        Parameters
        ----------
        nativeMediaType : str
            a string with the native media type, or ``None``

        """
        return _ogr.Feature_SetNativeMediaType(self, *args)

    def SetFieldString(self, *args):
        r"""
        SetFieldString(self, id, value)


        Set field to string value.

        :py:const:`OFTInteger` fields will be set based on an atoi() conversion of the
        string. :py:const:`OFTInteger64` fields will be set based on an :cpp:func:`CPLAtoGIntBig`
        conversion of the string. :py:const:`OFTReal` fields will be set based on an
        :cpp:func:`CPLAtof` conversion of the string. Other field types may be
        unaffected.

        See :cpp:func:`OGRFeature::SetField`.

        This method has only an effect on the in-memory feature object. If
        this object comes from a layer and the modifications must be
        serialized back to the datasource, :py:meth:`Layer.SetFeature` must be used
        afterwards. Or if this is a new feature, :py:meth:`Layer.CreateFeature` must be
        used afterwards.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.
        value : any
            the value to assign.

        """
        return _ogr.Feature_SetFieldString(self, *args)

    def Reference(self):
      pass

    def Dereference(self):
      pass

    def Destroy(self):
      "Once called, self has effectively been destroyed.  Do not access. For backwards compatibility only"
      _ogr.delete_Feature(self)
      self._invalidate_geom_refs()
      self.thisown = 0
      self.this = None

    def __cmp__(self, other):
        """Compares a feature to another for equality"""
        return self.Equal(other)

    def __copy__(self):
        return self.Clone()

    def _getfieldindex(self, fieldname):
        case_insensitive_idx = -1
        fdefn = _ogr.Feature_GetDefnRef(self)
        for i in range(fdefn.GetFieldCount()):
            name = fdefn.GetFieldDefn(i).GetName()
            if name == fieldname:
                return i
            elif case_insensitive_idx < 0 and name.lower() == fieldname.lower():
                case_insensitive_idx = i
        return case_insensitive_idx

    # This makes it possible to fetch fields in the form "feature.area".
    # This has some risk of name collisions.
    def __getattr__(self, key):
        """Returns the values of fields by the given name"""
        if key in ('this', 'thisown', '_geom_references'):
            return self.__getattribute__(key)

        idx = self._getfieldindex(key)
        if idx < 0:
            idx = self.GetGeomFieldIndex(key)
            if idx < 0:
                raise AttributeError(key)
            else:
                return self.GetGeomFieldRef(idx)
        else:
            return self.GetField(idx)

    # This makes it possible to set fields in the form "feature.area".
    # This has some risk of name collisions.
    def __setattr__(self, key, value):
        """Set the values of fields by the given name"""
        if key in ('this', 'thisown', '_geom_references'):
            super().__setattr__(key, value)
        else:
            idx = self._getfieldindex(key)
            if idx != -1:
                self._SetField2(idx, value)
            else:
                idx = self.GetGeomFieldIndex(key)
                if idx != -1:
                    self.SetGeomField(idx, value)
                else:
                    self.__dict__[key] = value

    # This makes it possible to fetch fields in the form "feature['area']".
    def __getitem__(self, key):
        """Returns the values of fields by the given name / field_index"""
        if isinstance(key, str):
            fld_index = self._getfieldindex(key)
        else:
            fld_index = key
            if key == self.GetFieldCount():
                raise IndexError
        if fld_index < 0:
            if isinstance(key, str):
                fld_index = self.GetGeomFieldIndex(key)
            if fld_index < 0:
                raise KeyError("Illegal field requested in GetField()")
            else:
                return self.GetGeomFieldRef(fld_index)
        else:
            return self.GetField(fld_index)

    # This makes it possible to set fields in the form "feature['area'] = 123".
    def __setitem__(self, key, value):
        """Returns the value of a field by field name / index"""
        if isinstance(key, str):
            fld_index = self._getfieldindex(key)
        else:
            fld_index = key
            if key == self.GetFieldCount():
                raise IndexError
        if fld_index < 0:
            if isinstance(key, str):
                fld_index = self.GetGeomFieldIndex(key)
            if fld_index < 0:
                raise KeyError("Illegal field requested in SetField()")
            else:
                return self.SetGeomField(fld_index, value)
        else:
            return self._SetField2(fld_index, value)

    def GetField(self, fld_index):
        """
        Get the value of a field in its native type.

        Alternatively, the ``[]`` operator may be used.

        Parameters
        ----------
        fld_index : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Examples
        --------
        >>> with gdal.OpenEx("poly.shp", gdal.OF_VECTOR) as ds:
        ...     lyr = ds.GetLayer(0)
        ...     feature = lyr.GetNextFeature()
        ...     # name-based access
        ...     feature.GetField("EAS_ID")
        ...     feature["EAS_ID"]
        ...     # index-based access
        ...     index = feature.GetFieldIndex("EAS_ID")
        ...     feature.GetField(index)
        ...     feature[index]
        ...
        168
        168
        168
        168
        """
        if isinstance(fld_index, str):
            fld_index = self._getfieldindex(fld_index)
        if (fld_index < 0) or (fld_index > self.GetFieldCount()):
            raise KeyError("Illegal field requested in GetField()")
        if not (self.IsFieldSet(fld_index)) or self.IsFieldNull(fld_index):
            return None
        fld_type = self.GetFieldType(fld_index)
        if fld_type == OFTInteger:
            if self.GetFieldDefnRef(fld_index).GetSubType() == OFSTBoolean:
                return bool(self.GetFieldAsInteger(fld_index))
            return self.GetFieldAsInteger(fld_index)
        if fld_type == OFTInteger64:
            return self.GetFieldAsInteger64(fld_index)
        if fld_type == OFTReal:
            return self.GetFieldAsDouble(fld_index)
        if fld_type == OFTStringList:
            return self.GetFieldAsStringList(fld_index)
        if fld_type == OFTIntegerList:
            ret = self.GetFieldAsIntegerList(fld_index)
            if self.GetFieldDefnRef(fld_index).GetSubType() == OFSTBoolean:
                 ret = [bool(x) for x in ret]
            return ret
        if fld_type == OFTInteger64List:
            return self.GetFieldAsInteger64List(fld_index)
        if fld_type == OFTRealList:
            return self.GetFieldAsDoubleList(fld_index)
    ## if fld_type == OFTDateTime or fld_type == OFTDate or fld_type == OFTTime:
    #     return self.GetFieldAsDate(fld_index)
    # default to returning as a string.  Should we add more types?
        try:
            return self.GetFieldAsString(fld_index)
        except:
    # For Python3 on non-UTF8 strings
            return self.GetFieldAsBinary(fld_index)

    def SetFieldBinary(self, field_index_or_name, value):
        """
        SetFieldBinary(Feature self, field_index_or_name: int | str, value: bytes)

        Set field to binary data.
        This function currently only has an effect on :py:const:`OFTBinary` fields.
        This function is the same as the C++ method :cpp:func:`OGRFeature::SetField`.

        Parameters
        ----------
        field_index_or_name : int or str
            the field to set, from 0 to GetFieldCount()-1. Or the field name
        values : any
            the data to apply.
        """

        if isinstance(field_index_or_name, str):
            fld_index = self._getfieldindex(field_index_or_name)
        else:
            fld_index = field_index_or_name
        self._SetFieldBinary(fld_index, value)

    def _SetField2(self, fld_index, value):
        if isinstance(fld_index, str):
            fld_index = self._getfieldindex(fld_index)
        if (fld_index < 0) or (fld_index > self.GetFieldCount()):
            raise KeyError("Illegal field requested in _SetField2()")

        if value is None:
            self.SetFieldNull(fld_index)
            return

        if isinstance(value, list):
            if not value:
                self.SetFieldNull(fld_index)
                return
            if isinstance(value[0], type(1)) or isinstance(value[0], type(12345678901234)):
                self.SetFieldInteger64List(fld_index, value)
                return
            elif isinstance(value[0], float):
                self.SetFieldDoubleList(fld_index, value)
                return
            elif isinstance(value[0], str):
                self.SetFieldStringList(fld_index, value)
                return
            else:
                raise TypeError('Unsupported type of list in _SetField2(). Type of element is %s' % str(type(value[0])))

        if isinstance(value, (bytes, bytearray, memoryview)) and self.GetFieldType(fld_index) == OFTBinary:
            self._SetFieldBinary(fld_index, value)
            return

        if hasattr(value, 'tolist'):
            self._SetField2(fld_index, value.tolist())
            return

        try:
            self.SetField(fld_index, value)
        except:
            self.SetField(fld_index, str(value))
        return

    def keys(self):
        """Return the list of field names (of the layer definition)"""
        names = []
        for i in range(self.GetFieldCount()):
            fieldname = self.GetFieldDefnRef(i).GetName()
            names.append(fieldname)
        return names

    def items(self):
        """Return a dictionary with the field names as key, and their value in the feature"""
        keys = self.keys()
        output = {}
        for key in keys:
            output[key] = self.GetField(key)
        return output

    def geometry(self):
        """ Return the feature geometry

            The lifetime of the returned geometry is bound to the one of its belonging
            feature.

            For more details: :cpp:func:`OGR_F_GetGeometryRef`

            The GetGeometryRef() method is also available as an alias of geometry()

            Returns
            -------
            Geometry or None
        """

        return self.GetGeometryRef()

    def __del__(self):
        self._invalidate_geom_refs()

    def __repr__(self):
        return self.DumpReadableAsString()


    def ExportToJson(self, as_object=False, options=None):
        """
        Export a GeoJSON object which represents the Feature.

        Parameters
        ----------
        as_object : bool, default = False
            determines whether the returned value should be a Python object instead of a string.
        options : dict or str
            Options to pass to :py:func:`Geometry.ExportToJson`

        Returns
        -------
        str or dict
        """
        try:
            import simplejson
        except ImportError:
            try:
                import json as simplejson
            except ImportError:
                raise ImportError("Unable to import simplejson or json, needed for ExportToJson.")

        geom = self.GetGeometryRef()
        if geom is not None:
            if options is None:
                options = []
            geom_json_string = geom.ExportToJson(options=options)
            geom_json_object = simplejson.loads(geom_json_string)
        else:
            geom_json_object = None

        output = {'type':'Feature',
                   'geometry': geom_json_object,
                   'properties': {}
                  }

        fid = self.GetFID()
        if fid != NullFID:
            output['id'] = fid

        for key in self.keys():
            fld_defn = self.GetFieldDefnRef(self.GetFieldIndex(key))
            if fld_defn.GetType() == _ogr.OFTInteger and fld_defn.GetSubType() == _ogr.OFSTBoolean:
                output['properties'][key] = bool(self.GetField(key))
            else:
                output['properties'][key] = self.GetField(key)

        if not as_object:
            output = simplejson.dumps(output)

        return output


    def _add_geom_ref(self, geom):
        if geom is None:
            return

        if not hasattr(self, '_geom_references'):
            import weakref

            self._geom_references = weakref.WeakSet()

        self._geom_references.add(geom)


    def _invalidate_geom_refs(self):
        if hasattr(self, '_geom_references'):
            for geom in self._geom_references:
                geom.this = None



# Register Feature in _ogr:
_ogr.Feature_swigregister(Feature)
class FeatureDefn(object):
    r"""


    Python proxy of an :cpp:class:`OGRFeatureDefn`.


    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_FeatureDefn

    def __init__(self, *args, **kwargs):
        r"""
        __init__(self, name_null_ok=None) -> FeatureDefn


        Create a new feature definition object to hold the field definitions.

        Parameters
        ----------
        name_null_ok : str, optional
            Name for the :py:class:`FeatureDefn`.

        """
        _ogr.FeatureDefn_swiginit(self, _ogr.new_FeatureDefn(*args, **kwargs))

    def GetName(self, *args):
        r"""
        GetName(self) -> char const *


        Get name of the :py:class:`FeatureDefn`.

        See :cpp:func:`OGRFeatureDefn::GetName`.

        Returns
        -------
        str
            the name

        """
        return _ogr.FeatureDefn_GetName(self, *args)

    def GetFieldCount(self, *args):
        r"""
        GetFieldCount(self) -> int


        Fetch number of fields on the passed feature definition.

        See :cpp:func:`OGRFeatureDefn::GetFieldCount`.

        Returns
        -------
        int
            count of fields.

        """
        return _ogr.FeatureDefn_GetFieldCount(self, *args)

    def GetFieldDefn(self, *args):
        r"""
        GetFieldDefn(self, i) -> FieldDefn


        Fetch field definition of the passed feature definition.

        See :cpp:func:`OGRFeatureDefn::GetFieldDefn`.

        Parameters
        ----------
        i : int or str
            Field name or 0-based numeric index. For repeated
            access, use of the numeric index avoids a lookup
            step.

        Returns
        -------
        FieldDefn
            internal field definition object or ``None`` if the field does not
            exist. This object should not be modified by the application.

        """

        if type(args[0]) is str:
            args = (self.GetFieldIndex(args[0]), )


        return _ogr.FeatureDefn_GetFieldDefn(self, *args)


    def GetFieldIndex(self, *args):
        r"""
        GetFieldIndex(self, field_name) -> int


        Find field by name.

        The field index of the first field matching the passed field name
        (case insensitively) is returned.

        See :cpp:func:`OGRFeatureDefn::GetFieldIndex`.

        Parameters
        ----------
        field_name : str
            the field name to search for.

        Returns
        -------
        int
            the field index, or -1 if no match found.

        """
        return _ogr.FeatureDefn_GetFieldIndex(self, *args)

    def AddFieldDefn(self, *args):
        r"""
        AddFieldDefn(self, defn)


        Add a new field definition.

        To add a new field definition to a layer definition, do not use this
        function directly, but use :py:meth:`Layer.CreateField` instead.

        This function should only be called while there are no :py:class:`Feature`
        objects in existence based on this :py:class:`FeatureDefn`. The
        :py:class:`FieldDefn` passed in is copied.

        See :cpp:func:`OGRFeatureDefn::AddFieldDefn`.

        Parameters
        ----------
        defn : FieldDefn
            the new field definition.

        """
        return _ogr.FeatureDefn_AddFieldDefn(self, *args)

    def GetGeomFieldCount(self, *args):
        r"""
        GetGeomFieldCount(self) -> int


        Fetch number of geometry fields on the passed feature definition.

        See :cpp:func:`OGRFeatureDefn::GetGeomFieldCount`.

        Returns
        -------
        int
            count of geometry fields.

        """
        return _ogr.FeatureDefn_GetGeomFieldCount(self, *args)

    def GetGeomFieldDefn(self, *args):
        r"""
        GetGeomFieldDefn(self, i) -> GeomFieldDefn


        Fetch geometry field definition of the passed feature definition.

        See :cpp:func:`OGRFeatureDefn::GetGeomFieldDefn`.

        Parameters
        ----------
        i : int
            the geometry field to fetch, between 0 and GetGeomFieldCount() - 1.

        Returns
        -------
        GeomFieldDefn
            an internal field definition object or ``None`` if invalid
            index. This object should not be modified by the application.

        """
        return _ogr.FeatureDefn_GetGeomFieldDefn(self, *args)

    def GetGeomFieldIndex(self, *args):
        r"""
        GetGeomFieldIndex(self, field_name) -> int


        Find geometry field by name.

        The geometry field index of the first geometry field matching the
        passed field name (case insensitively) is returned.

        See :cpp:func:`OGRFeatureDefn::GetGeomFieldIndex`.

        Parameters
        ----------
        field_name : str
            the geometry field name to search for.

        Returns
        -------
        int
            the geometry field index, or -1 if no match found.

        """
        return _ogr.FeatureDefn_GetGeomFieldIndex(self, *args)

    def AddGeomFieldDefn(self, *args):
        r"""
        AddGeomFieldDefn(self, defn)


        Add a new geometry field definition.

        To add a new field definition to a layer definition, do not use this
        function directly, but use :py:meth:`Layer.CreateGeomField` instead.

        This function should only be called while there are no :py:class:`Feature`
        objects in existence based on this :py:class:`FeatureDefn`. The
        :py:class:`GeomFieldDefn` passed in is copied.

        See :cpp:Func:`OGRFeatureDefn::AddGeomFieldDefn`.

        Parameters
        ----------
        defn : GeomFieldDefn
            new geometry field definition.

        """
        return _ogr.FeatureDefn_AddGeomFieldDefn(self, *args)

    def DeleteGeomFieldDefn(self, *args):
        r"""
        DeleteGeomFieldDefn(self, idx) -> OGRErr


        Delete an existing geometry field definition.

        To delete an existing geometry field definition from a layer
        definition, do not use this function directly, but use
        :py:meth:`Layer.DeleteGeomField` instead ( not implemented yet).

        This function should only be called while there are no :py:class:`Feature`
        objects in existence based on this :py:class:`FeatureDefn`.

        See :cpp:func:`OGRFeatureDefn::DeleteGeomFieldDefn`.

        Parameters
        ----------
        idx : int
            the index of the geometry field definition.

        Returns
        -------
        int
            :py:const:`OGRERR_NONE` in case of success.

        """
        return _ogr.FeatureDefn_DeleteGeomFieldDefn(self, *args)

    def GetGeomType(self, *args):
        r"""
        GetGeomType(self) -> OGRwkbGeometryType


        Fetch the geometry base type of the passed feature definition.

        This is equivalent to ``GetGeomFieldDefn(0).GetType()``.

        See :cpp:func:`OGRFeatureDefn::GetGeomType`.

        Returns
        -------
        int
            the base type for all geometry related to this definition.

        """
        return _ogr.FeatureDefn_GetGeomType(self, *args)

    def SetGeomType(self, *args):
        r"""
        SetGeomType(self, geom_type)


        Assign the base geometry type for the passed layer (the same as the
        feature definition).

        This is equivalent to ``GetGeomFieldDefn(0).SetType()``.

        All geometry objects using this type must be of the defined type or a
        derived type. The default upon creation is :py:const:`wkbUnknown` which allows for
        any geometry type. The geometry type should generally not be changed
        after any :py:class:`Feature` objects have been created against this definition.

        See :cpp:func:`OGRFeatureDefn::SetGeomType`.

        Parameters
        ----------
        geom_type : int
            the new type to assign.

        """
        return _ogr.FeatureDefn_SetGeomType(self, *args)

    def GetReferenceCount(self, *args):
        r"""
        GetReferenceCount(self) -> int


        Fetch current reference count.

        See :cpp:func:`OGRFeatureDefn::GetReferenceCount`.

        Returns
        -------
        int
            the current reference count.

        """
        return _ogr.FeatureDefn_GetReferenceCount(self, *args)

    def IsGeometryIgnored(self, *args):
        r"""
        IsGeometryIgnored(self) -> int


        Determine whether the geometry can be omitted when fetching features.

        Equivalent to ``GetGeomFieldDefn(0).IsIgnored()``.

        See :cpp:func:`OGRFeatureDefn::IsGeometryIgnored`.

        Returns
        -------
        int
            ignore state

        """
        return _ogr.FeatureDefn_IsGeometryIgnored(self, *args)

    def SetGeometryIgnored(self, *args):
        r"""
        SetGeometryIgnored(self, bIgnored)


        Set whether the geometry can be omitted when fetching features.

        This is equivalent to ``GetGeomFieldDefn(0).SetIgnored()``.

        See :cpp:func:`OGRFeatureDefn::SetGeometryIgnored`.

        Parameters
        ----------
        bignored : bool
            ignore state

        """
        return _ogr.FeatureDefn_SetGeometryIgnored(self, *args)

    def IsStyleIgnored(self, *args):
        r"""
        IsStyleIgnored(self) -> int


        Determine whether the style can be omitted when fetching features.

        See :cpp:func:`OGRFeatureDefn::IsStyleIgnored`.

        Returns
        -------
        int
            ignore state

        """
        return _ogr.FeatureDefn_IsStyleIgnored(self, *args)

    def SetStyleIgnored(self, *args):
        r"""
        SetStyleIgnored(self, bIgnored)


        Set whether the style can be omitted when fetching features.

        See :cpp:func:`OGRFeatureDefn::SetStyleIgnored`.

        Parameters
        ----------
        bignored : bool
            ignore state

        """
        return _ogr.FeatureDefn_SetStyleIgnored(self, *args)

    def IsSame(self, *args):
        r"""
        IsSame(self, other_defn) -> int


        Test if the feature definition is identical to the other one.

        Parameters
        ----------
        other_defn : FeatureDefn
            other feature definition to compare to.

        Returns
        -------
        int
            1 if the feature definition is identical to the other one.

        """
        return _ogr.FeatureDefn_IsSame(self, *args)

    def Destroy(self):
      "Once called, self has effectively been destroyed.  Do not access. For backwards compatibility only"
      _ogr.delete_FeatureDefn(self)
      self.thisown = 0



# Register FeatureDefn in _ogr:
_ogr.FeatureDefn_swigregister(FeatureDefn)
class FieldDefn(object):
    r"""

    Python proxy of an :cpp:class:`OGRFieldDefn`.

    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_FieldDefn

    def __init__(self, *args, **kwargs):
        r"""
        __init__(self, name_null_ok="unnamed", field_type=OFTString) -> FieldDefn

        Python proxy of an :cpp:class:`OGRFieldDefn`.

        """
        _ogr.FieldDefn_swiginit(self, _ogr.new_FieldDefn(*args, **kwargs))

    def GetName(self, *args):
        r"""GetName(self) -> char const *"""
        return _ogr.FieldDefn_GetName(self, *args)

    def GetNameRef(self, *args):
        r"""
        GetNameRef(self) -> char const *


        Fetch name of this field.

        See :cpp:func:`OGRFieldDefn::GetNameRef`.

        Returns
        -------
        str
            the name of the field definition.

        """
        return _ogr.FieldDefn_GetNameRef(self, *args)

    def SetName(self, *args):
        r"""
        SetName(self, name)


        Reset the name of this field.

        See :cpp:func:`OGRFieldDefn::SetName`.

        Parameters
        ----------
        name : str
            the new name to apply

        """
        return _ogr.FieldDefn_SetName(self, *args)

    def GetAlternativeName(self, *args):
        r"""GetAlternativeName(self) -> char const *"""
        return _ogr.FieldDefn_GetAlternativeName(self, *args)

    def GetAlternativeNameRef(self, *args):
        r"""
        GetAlternativeNameRef(self) -> char const *


        Fetch the alternative name (or "alias") for this field.

        The alternative name is an optional attribute for a field which can
        provide a more user-friendly, descriptive name of a field which is not
        subject to the usual naming constraints defined by the data provider.

        This is a metadata style attribute only: the alternative name cannot
        be used in place of the actual field name during SQL queries or other
        field name dependent API calls.

        See :cpp:func:`OGRFieldDefn::GetAlternativeNameRef`.

        .. versionadded:: 3.2

        Returns
        -------
        str
            the alternative name of the field definition.

        """
        return _ogr.FieldDefn_GetAlternativeNameRef(self, *args)

    def SetAlternativeName(self, *args):
        r"""
        SetAlternativeName(self, alternativeName)


        Reset the alternative name (or "alias") for this field.

        The alternative name is an optional attribute for a field which can
        provide a more user-friendly, descriptive name of a field which is not
        subject to the usual naming constraints defined by the data provider.

        This is a metadata style attribute only: the alternative name cannot
        be used in place of the actual field name during SQL queries or other
        field name dependent API calls.

        See :cpp:func:`OGRFieldDefn::SetAlternativeName`.

        .. versionadded:: 3.2

        Parameters
        ----------
        alternativeName : str
            the new alternative name to apply.

        """
        return _ogr.FieldDefn_SetAlternativeName(self, *args)

    def GetType(self, *args):
        r"""
        GetType(self) -> OGRFieldType


        Fetch type of this field.

        See :cpp:func:`OGRFieldDefn::GetType`.

        Returns
        -------
        int
            field type code, e.g. :py:const:`OFTInteger`

        """
        return _ogr.FieldDefn_GetType(self, *args)

    def SetType(self, *args):
        r"""
        SetType(self, type)


        Set the type of this field.

        This should never be done to a :py:class:`FieldDefn` that is already part of
        an :py:class:`FeatureDefn`.

        See :cpp:func:`OGRFieldDefn::SetType`.

        Parameters
        ----------
        type : int
            the new field type.

        Examples
        --------
        >>> f = ogr.FieldDefn()
        >>> f.SetType(ogr.OFTReal)

        """
        return _ogr.FieldDefn_SetType(self, *args)

    def GetSubType(self, *args):
        r"""
        GetSubType(self) -> OGRFieldSubType


        Fetch subtype of this field.

        See :cpp:func:`OGRFieldDefn::GetSubType`.

        Returns
        -------
        int
            field subtype code, default = :py:const:`OFSTNone`

        """
        return _ogr.FieldDefn_GetSubType(self, *args)

    def SetSubType(self, *args):
        r"""
        SetSubType(self, type)


        Set the subtype of this field.

        This should never be done to a :py:class:`FieldDefn` that is already part of
        an :py:class:FeatureDefn`.

        See :cpp:func:`OGRFieldDefn::SetSubType`.

        Parameters
        ----------
        type :
            the new field subtype.

        Examples
        --------
        >>> f = ogr.FieldDefn()
        >>> f.SetType(ogr.OFTReal)
        >>> f.SetSubType(ogr.OFSTJSON)
        Warning 1: Type and subtype of field definition are not compatible. Resetting to OFSTNone
        >>> f.SetSubType(ogr.OFSTFloat32)

        """
        return _ogr.FieldDefn_SetSubType(self, *args)

    def GetJustify(self, *args):
        r"""
        GetJustify(self) -> OGRJustification


        Get the justification for this field.

        See :cpp:func:`OGRFieldDefn::GetJustify`.

        Note: no driver is know to use the concept of field justification.

        Returns
        -------
        int
            the justification.

        """
        return _ogr.FieldDefn_GetJustify(self, *args)

    def SetJustify(self, *args):
        r"""
        SetJustify(self, justify)


        Set the justification for this field.

        Note: no driver is know to use the concept of field justification.

        See :cpp:func:`OGRFieldDefn::SetJustify`.

        Parameters
        ----------
        justify : int
            the new justification

        Examples
        --------
        >>> f = ogr.FieldDefn('desc', ogr.OFTString)
        >>> f.SetJustify(ogr.OJRight)

        """
        return _ogr.FieldDefn_SetJustify(self, *args)

    def GetWidth(self, *args):
        r"""
        GetWidth(self) -> int


        Get the formatting width for this field.

        See :cpp:func:`OGRFieldDefn::GetWidth`.

        Returns
        -------
        int
            the width, zero means no specified width.

        """
        return _ogr.FieldDefn_GetWidth(self, *args)

    def SetWidth(self, *args):
        r"""
        SetWidth(self, width)


        Set the formatting width for this field in characters.

        See :cpp:func:`OGRFieldDefn::SetWidth`.

        Parameters
        ----------
        width : int
            the new width

        """
        return _ogr.FieldDefn_SetWidth(self, *args)

    def GetPrecision(self, *args):
        r"""
        GetPrecision(self) -> int


        Get the formatting precision for this field.

        This should normally be zero for fields of types other than :py:const:`OFTReal`.

        See :cpp:func:`OGRFieldDefn::GetPrecision`.

        Returns
        -------
        int
            the precision.

        """
        return _ogr.FieldDefn_GetPrecision(self, *args)

    def SetPrecision(self, *args):
        r"""
        SetPrecision(self, precision)


        Set the formatting precision for this field in characters.

        This should normally be zero for fields of types other than :py:const:`OFTReal`.

        See :cpp:func:`OGRFieldDefn::SetPrecision`.

        Parameters
        ----------
        precision : int
            the new precision.

        """
        return _ogr.FieldDefn_SetPrecision(self, *args)

    def GetTZFlag(self, *args):
        r"""GetTZFlag(self) -> int"""
        return _ogr.FieldDefn_GetTZFlag(self, *args)

    def SetTZFlag(self, *args):
        r"""SetTZFlag(self, tzflag)"""
        return _ogr.FieldDefn_SetTZFlag(self, *args)

    def GetTypeName(self, *args):
        r"""GetTypeName(self) -> char const *"""
        return _ogr.FieldDefn_GetTypeName(self, *args)

    def GetFieldTypeName(self, *args):
        r"""GetFieldTypeName(self, type) -> char const *"""
        return _ogr.FieldDefn_GetFieldTypeName(self, *args)

    def IsIgnored(self, *args):
        r"""
        IsIgnored(self) -> int


        Return whether this field should be omitted when fetching features.

        See :cpp:func:`OGRFieldDefn::IsIgnored`.

        Returns
        -------
        int
            ignore state

        """
        return _ogr.FieldDefn_IsIgnored(self, *args)

    def SetIgnored(self, *args):
        r"""
        SetIgnored(self, bIgnored)


        Set whether this field should be omitted when fetching features.

        See :cpp:func:`OGRFieldDefn::SetIgnored`.

        Parameters
        ----------
        bignored : bool
            ignore state

        """
        return _ogr.FieldDefn_SetIgnored(self, *args)

    def IsNullable(self, *args):
        r"""
        IsNullable(self) -> int


        Return whether this field can receive null values.

        By default, fields are nullable.

        Even if this method returns FALSE (i.e not-nullable field), it doesn't
        mean that :py:meth:`Feature.IsFieldSet` will necessary return TRUE, as
        fields can be temporary unset and null/not-null validation is usually
        done when :py:meth:`Layer.CreateFeature`/:py:meth:`Layer.SetFeature` is called.

        See :cpp:func:`OGRFieldDefn::IsNullable`.

        Returns
        -------
        int
            TRUE if the field is authorized to be null.

        """
        return _ogr.FieldDefn_IsNullable(self, *args)

    def SetNullable(self, *args):
        r"""
        SetNullable(self, bNullable)


        Set whether this field can receive null values.

        By default, fields are nullable, so this method is generally called
        with ``False`` to set a not-null constraint.

        Drivers that support writing not-null constraint will advertise the
        ``GDAL_DCAP_NOTNULL_FIELDS`` driver metadata item.

        See :cpp:func:`OGRFieldDefn::SetNullable`.

        Parameters
        ----------
        bNullable : bool
            ``False`` if the field must have a not-null constraint.

        """
        return _ogr.FieldDefn_SetNullable(self, *args)

    def IsUnique(self, *args):
        r"""
        IsUnique(self) -> int


        Return whether this field has a unique constraint.

        By default, fields have no unique constraint.

        See :cpp:func:`OGRFieldDefn::IsUnique`.

        .. versionadded:: 3.2

        Returns
        -------
        int
            TRUE if the field has a unique constraint.

        """
        return _ogr.FieldDefn_IsUnique(self, *args)

    def SetUnique(self, *args):
        r"""
        SetUnique(self, bUnique)


        Set whether this field has a unique constraint.

        By default, fields have no unique constraint, so this method is
        generally called with TRUE to set a unique constraint.

        Drivers that support writing unique constraint will advertise the
        ``GDAL_DCAP_UNIQUE_FIELDS`` driver metadata item.

        Note that once a :py:class:`FieldDefn` has been added to a layer definition with
        :py:meth:`Layer.AddFieldDefn`, its setter methods should not be called on the
        object returned with ``GetLayerDefn().GetFieldDefn()``. Instead,
        :py:meth:`Layer::AlterFieldDefn` should be called on a new instance of
        :py:class:`FieldDefn`, for drivers that support :py:meth:`Layer.AlterFieldDefn`.

        See :cpp:func:`OGRFieldDefn::SetUnique`.

        .. versionadded:: 3.2

        Parameters
        ----------
        bUnique : bool
            ``True`` if the field must have a unique constraint

        """
        return _ogr.FieldDefn_SetUnique(self, *args)

    def IsGenerated(self, *args):
        r"""IsGenerated(self) -> int"""
        return _ogr.FieldDefn_IsGenerated(self, *args)

    def SetGenerated(self, *args):
        r"""SetGenerated(self, bGenerated)"""
        return _ogr.FieldDefn_SetGenerated(self, *args)

    def GetDefault(self, *args):
        r"""
        GetDefault(self) -> char const *


        Get default field value.

        See :cpp:func:`OGRFieldDefn::GetDefault`.

        Returns
        -------
        str
            default field value or ``None``.

        """
        return _ogr.FieldDefn_GetDefault(self, *args)

    def SetDefault(self, *args):
        r"""
        SetDefault(self, pszValue)


        Set default field value.

        The default field value is taken into account by drivers (generally
        those with a SQL interface) that support it at field creation time.
        OGR will generally not automatically set the default field value to
        null fields by itself when calling OGRFeature::CreateFeature() /
        OGRFeature::SetFeature(), but will let the low-level layers to do the
        job. So retrieving the feature from the layer is recommended.

        The accepted values are NULL, a numeric value, a literal value
        enclosed between single quote characters (and inner single quote
        characters escaped by repetition of the single quote character),
        CURRENT_TIMESTAMP, CURRENT_TIME, CURRENT_DATE or a driver specific
        expression (that might be ignored by other drivers). For a datetime
        literal value, format should be 'YYYY/MM/DD HH:MM:SS[.sss]'
        (considered as UTC time).

        Drivers that support writing DEFAULT clauses will advertise the
        GDAL_DCAP_DEFAULT_FIELDS driver metadata item.

        See :cpp:func:`OGRFieldDefn::SetDefault`.

        Parameters
        ----------
        pszValue : str
            new default field value or NULL pointer.

        """
        return _ogr.FieldDefn_SetDefault(self, *args)

    def IsDefaultDriverSpecific(self, *args):
        r"""
        IsDefaultDriverSpecific(self) -> int


        Returns whether the default value is driver specific.

        Driver specific default values are those that are not NULL, a numeric
        value, a literal value enclosed between single quote characters,
        CURRENT_TIMESTAMP, CURRENT_TIME, CURRENT_DATE or datetime literal
        value.

        See :cpp:func:`OGRFieldDefn::IsDefaultDriverSpecific`.

        Returns
        -------
        int
            TRUE if the default value is driver specific.

        """
        return _ogr.FieldDefn_IsDefaultDriverSpecific(self, *args)

    def GetDomainName(self, *args):
        r"""
        GetDomainName(self) -> char const *


        Return the name of the field domain for this field.

        By default an empty string is returned.

        Field domains ( :py:class:`FieldDomain` class) are attached at the :py:class:`Dataset` level and should be retrieved with :py:meth:`Dataset.GetFieldDomain`.

        See :cpp:func:`OGRFieldDefn::GetDomainName`.

        .. versionadded:: 3.3

        Returns
        -------
        str
            the field domain name, or an empty string if there is none.

        """
        return _ogr.FieldDefn_GetDomainName(self, *args)

    def SetDomainName(self, *args):
        r"""
        SetDomainName(self, name)


        Set the name of the field domain for this field.

        Field domains ( :py:class:`FieldDomain`) are attached at the :py:class:`Dataset` level.

        See :cpp:func:`OGRFieldDefn::SetDomainName`.

        .. versionadded:: 3.3

        Parameters
        ----------
        name : str
            Field domain name.

        """
        return _ogr.FieldDefn_SetDomainName(self, *args)

    def GetComment(self, *args):
        r"""GetComment(self) -> char const *"""
        return _ogr.FieldDefn_GetComment(self, *args)

    def SetComment(self, *args):
        r"""SetComment(self, comment)"""
        return _ogr.FieldDefn_SetComment(self, *args)

    width = property(GetWidth, SetWidth)
    type = property(GetType, SetType)
    precision = property(GetPrecision, SetPrecision)
    name = property(GetName, SetName)
    justify = property(GetJustify, SetJustify)


    def Destroy(self):
      "Once called, self has effectively been destroyed.  Do not access. For backwards compatibility only"
      _ogr.delete_FieldDefn(self)
      self.thisown = 0


# Register FieldDefn in _ogr:
_ogr.FieldDefn_swigregister(FieldDefn)
class GeomFieldDefn(object):
    r"""Proxy of C++ OGRGeomFieldDefnShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_GeomFieldDefn

    def __init__(self, *args, **kwargs):
        r"""__init__(self, name_null_ok="", field_type=wkbUnknown) -> GeomFieldDefn"""
        _ogr.GeomFieldDefn_swiginit(self, _ogr.new_GeomFieldDefn(*args, **kwargs))

    def GetName(self, *args):
        r"""GetName(self) -> char const *"""
        return _ogr.GeomFieldDefn_GetName(self, *args)

    def GetNameRef(self, *args):
        r"""GetNameRef(self) -> char const *"""
        return _ogr.GeomFieldDefn_GetNameRef(self, *args)

    def SetName(self, *args):
        r"""SetName(self, name)"""
        return _ogr.GeomFieldDefn_SetName(self, *args)

    def GetType(self, *args):
        r"""GetType(self) -> OGRwkbGeometryType"""
        return _ogr.GeomFieldDefn_GetType(self, *args)

    def SetType(self, *args):
        r"""SetType(self, type)"""
        return _ogr.GeomFieldDefn_SetType(self, *args)

    def GetSpatialRef(self, *args):
        r"""GetSpatialRef(self) -> SpatialReference"""
        return _ogr.GeomFieldDefn_GetSpatialRef(self, *args)

    def SetSpatialRef(self, *args):
        r"""SetSpatialRef(self, srs)"""
        return _ogr.GeomFieldDefn_SetSpatialRef(self, *args)

    def IsIgnored(self, *args):
        r"""IsIgnored(self) -> int"""
        return _ogr.GeomFieldDefn_IsIgnored(self, *args)

    def SetIgnored(self, *args):
        r"""SetIgnored(self, bIgnored)"""
        return _ogr.GeomFieldDefn_SetIgnored(self, *args)

    def IsNullable(self, *args):
        r"""IsNullable(self) -> int"""
        return _ogr.GeomFieldDefn_IsNullable(self, *args)

    def SetNullable(self, *args):
        r"""SetNullable(self, bNullable)"""
        return _ogr.GeomFieldDefn_SetNullable(self, *args)

    def GetCoordinatePrecision(self, *args):
        r"""GetCoordinatePrecision(self) -> GeomCoordinatePrecision"""
        return _ogr.GeomFieldDefn_GetCoordinatePrecision(self, *args)

    def SetCoordinatePrecision(self, *args):
        r"""SetCoordinatePrecision(self, coordPrec)"""
        return _ogr.GeomFieldDefn_SetCoordinatePrecision(self, *args)

    type = property(GetType, SetType)
    name = property(GetName, SetName)
    srs = property(GetSpatialRef, SetSpatialRef)


# Register GeomFieldDefn in _ogr:
_ogr.GeomFieldDefn_swigregister(GeomFieldDefn)

def CreateGeometryFromWkb(*args, **kwargs):
    r"""CreateGeometryFromWkb(len, reference=None) -> Geometry"""
    return _ogr.CreateGeometryFromWkb(*args, **kwargs)

def CreateGeometryFromWkt(*args, **kwargs):
    r"""CreateGeometryFromWkt(val, reference=None) -> Geometry"""
    return _ogr.CreateGeometryFromWkt(*args, **kwargs)

def CreateGeometryFromGML(*args):
    r"""CreateGeometryFromGML(input_string) -> Geometry"""
    return _ogr.CreateGeometryFromGML(*args)

def CreateGeometryFromJson(*args):
    r"""CreateGeometryFromJson(input_string) -> Geometry"""
    return _ogr.CreateGeometryFromJson(*args)

def CreateGeometryFromEsriJson(*args):
    r"""CreateGeometryFromEsriJson(input_string) -> Geometry"""
    return _ogr.CreateGeometryFromEsriJson(*args)

def CreateGeometryFromEnvelope(*args):
    r"""
    CreateGeometryFromEnvelope(xmin, ymin, xmax, ymax, reference=None) -> Geometry


    Creates a rectangular, axis-aligned Polygon geometry.

    .. versionadded:: 3.12

    Parameters
    ----------
    xmin : float
        minimum x coordinate value
    ymin : float
        minimum y coordinate value
    xmax : float
        maximum x coordinate value
    ymax : float
        maximum y coordinate value
    srs : SpatialReference, optional
        optional spatial reference to assign to the geometry

    Returns
    -------
    Geometry

    Examples
    --------
    >>> g = ogr.CreateGeometryFromEnvelope(424788, 25211, 581555, 279799)
    >>> g.ExportToWkt()
    'POLYGON ((424788 25211,424788 279799,581555 279799,581555 25211,424788 25211))'

    """
    return _ogr.CreateGeometryFromEnvelope(*args)

def BuildPolygonFromEdges(*args, **kwargs):
    r"""BuildPolygonFromEdges(hLineCollection, bBestEffort=0, bAutoClose=0, dfTolerance=0) -> Geometry"""
    return _ogr.BuildPolygonFromEdges(*args, **kwargs)

def ApproximateArcAngles(*args, **kwargs):
    r"""ApproximateArcAngles(dfCenterX, dfCenterY, dfZ, dfPrimaryRadius, dfSecondaryAxis, dfRotation, dfStartAngle, dfEndAngle, dfMaxAngleStepSizeDegrees) -> Geometry"""
    return _ogr.ApproximateArcAngles(*args, **kwargs)

def ForceToPolygon(*args):
    r"""ForceToPolygon(geom_in) -> Geometry"""
    return _ogr.ForceToPolygon(*args)

def ForceToLineString(*args):
    r"""ForceToLineString(geom_in) -> Geometry"""
    return _ogr.ForceToLineString(*args)

def ForceToMultiPolygon(*args):
    r"""ForceToMultiPolygon(geom_in) -> Geometry"""
    return _ogr.ForceToMultiPolygon(*args)

def ForceToMultiPoint(*args):
    r"""ForceToMultiPoint(geom_in) -> Geometry"""
    return _ogr.ForceToMultiPoint(*args)

def ForceToMultiLineString(*args):
    r"""ForceToMultiLineString(geom_in) -> Geometry"""
    return _ogr.ForceToMultiLineString(*args)

def ForceTo(*args):
    r"""ForceTo(geom_in, eTargetType, options=None) -> Geometry"""
    return _ogr.ForceTo(*args)
class Geometry(object):
    r"""Proxy of C++ OGRGeometryShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_Geometry

    def __init__(self, *args, **kwargs):
        r"""__init__(self, type=wkbUnknown, wkt=None, wkb=0, gml=None) -> Geometry"""
        _ogr.Geometry_swiginit(self, _ogr.new_Geometry(*args, **kwargs))

    def ExportToWkt(self, *args):
        r"""
        ExportToWkt(self) -> OGRErr

        Convert a geometry into well known text format.

        For more details: :cpp:func:`OGR_G_ExportToWkt`

        Returns
        -------
        str

        """
        return _ogr.Geometry_ExportToWkt(self, *args)

    def ExportToIsoWkt(self, *args):
        r"""
        ExportToIsoWkt(self) -> OGRErr

        Convert a geometry into SFSQL 1.2 / ISO SQL/MM Part 3 well known text
        format.

        For more details: :cpp:func:`OGR_G_ExportToIsoWkt`

        .. versionadded:: 2.0

        Returns
        -------
        str

        """
        return _ogr.Geometry_ExportToIsoWkt(self, *args)

    def ExportToWkb(self, *args, **kwargs):
        r"""
        ExportToWkb(self, byte_order=wkbNDR) -> OGRErr

        Convert a geometry well known binary format.

        For more details: :cpp:func:`OGR_G_ExportToWkb`

        Parameters
        ----------
        byte_order : int, optional
            One of :py:const:`osgeo.ogr.wkbXDR` or :py:const:`osgeo.ogr.wkbNDR`,
            indicating MSB (most significant byte) or LSB (least significant byte)
            byte order. Defaults to :py:const:`osgeo.ogr.wkbNDR`.

        Returns
        -------
        bytes

        """
        return _ogr.Geometry_ExportToWkb(self, *args, **kwargs)

    def ExportToIsoWkb(self, *args, **kwargs):
        r"""
        ExportToIsoWkb(self, byte_order=wkbNDR) -> OGRErr

        Convert a geometry into SFSQL 1.2 / ISO SQL/MM Part 3 well known
        binary format.

        For more details: :cpp:func:`OGR_G_ExportToIsoWkb`

        .. versionadded:: 2.0

        Parameters
        ----------
        byte_order : int, optional
            One of :py:const:`osgeo.ogr.wkbXDR` or :py:const:`osgeo.ogr.wkbNDR`,
            indicating MSB (most significant byte) or LSB (least significant byte)
            byte order. Defaults to :py:const:`osgeo.ogr.wkbNDR`.

        Returns
        -------
        bytes

        """
        return _ogr.Geometry_ExportToIsoWkb(self, *args, **kwargs)

    def ExportToGML(self, *args, **kwargs):
        r"""ExportToGML(self, options=None) -> retStringAndCPLFree *"""
        return _ogr.Geometry_ExportToGML(self, *args, **kwargs)

    def ExportToKML(self, *args):
        r"""ExportToKML(self, altitude_mode=None) -> retStringAndCPLFree *"""
        return _ogr.Geometry_ExportToKML(self, *args)

    def ExportToJson(self, *args, **kwargs):
        r"""ExportToJson(self, options=None) -> retStringAndCPLFree *"""
        return _ogr.Geometry_ExportToJson(self, *args, **kwargs)

    def AddPoint(self, *args, **kwargs):
        r"""
        AddPoint(self, x, y, z=0) -> OGRErr

        Add a point to a geometry (line string or point).

        The vertex count of the line string is increased by one, and assigned from
        the passed location value.

        The geometry is promoted to include a Z component, if it does not already
        have one, even if the Z parameter is not explicitly specified. To avoid that
        use AddPoint_2D.

        This is the same as :cpp:func:`OGR_G_AddPoint`

        Parameters
        ----------
        X : float
            x coordinate of point to add.
        Y : float
            y coordinate of point to add.
        Z : float
            z coordinate of point to add. Defaults to 0

        Examples
        --------
        >>> pt = ogr.Geometry(ogr.wkbPoint)
        >>> ogr.GeometryTypeToName(pt.GetGeometryType())
        'Point'
        >>> pt.AddPoint(3, 7)
        0
        >>> ogr.GeometryTypeToName(pt.GetGeometryType())
        '3D Point'

        """
        return _ogr.Geometry_AddPoint(self, *args, **kwargs)

    def AddPointM(self, *args, **kwargs):
        r"""AddPointM(self, x, y, m) -> OGRErr"""
        return _ogr.Geometry_AddPointM(self, *args, **kwargs)

    def AddPointZM(self, *args, **kwargs):
        r"""AddPointZM(self, x, y, z, m) -> OGRErr"""
        return _ogr.Geometry_AddPointZM(self, *args, **kwargs)

    def AddPoint_2D(self, *args):
        r"""
        AddPoint_2D(self, x, y) -> OGRErr

        Add a point to a geometry (line string or point).

        The vertex count of the line string is increased by one, and assigned from
        the passed location value.

        If the geometry includes a Z or M component, the value for those components
        for the added point will be 0.

        This is the same as :cpp:func:`OGR_G_AddPoint_2D`

        Parameters
        ----------
        X : float
            x coordinate of point to add.
        Y : float
            y coordinate of point to add.

        Examples
        --------
        >>> pt = ogr.Geometry(ogr.wkbPoint)
        >>> ogr.GeometryTypeToName(pt.GetGeometryType())
        'Point'
        >>> pt.AddPoint_2D(3, 7)
        0
        >>> ogr.GeometryTypeToName(pt.GetGeometryType())
        'Point'

        """
        return _ogr.Geometry_AddPoint_2D(self, *args)

    def AddGeometryDirectly(self, *args):
        r"""AddGeometryDirectly(self, other_disown) -> OGRErr"""
        return _ogr.Geometry_AddGeometryDirectly(self, *args)

    def AddGeometry(self, *args):
        r"""AddGeometry(self, other) -> OGRErr"""
        return _ogr.Geometry_AddGeometry(self, *args)

    def RemoveGeometry(self, *args):
        r"""RemoveGeometry(self, iSubGeom) -> OGRErr"""
        return _ogr.Geometry_RemoveGeometry(self, *args)

    def Clone(self, *args):
        r"""
        Clone(self) -> Geometry

        Make a copy of this object.

        For more details: :cpp:func:`OGR_G_Clone`

        Returns
        -------
        Geometry
            The copy of the geometry with the same spatial reference system
            as the original.

        """
        return _ogr.Geometry_Clone(self, *args)

    def GetGeometryType(self, *args):
        r"""
        GetGeometryType(self) -> OGRwkbGeometryType

        Fetch geometry type.

        For more details: :cpp:func:`OGR_G_GetGeometryType`

        Returns
        -------
        int
            The geometry type code. The types can be found with
            'osgeo.ogr.wkb' prefix. For example :py:const:`osgeo.ogr.wkbPolygon`.

        """
        return _ogr.Geometry_GetGeometryType(self, *args)

    def GetGeometryName(self, *args):
        r"""
        GetGeometryName(self) -> char const *

        Fetch WKT name for geometry type.

        For more details: :cpp:func:`OGR_G_GetGeometryName`

        geometry to get name from.

        Returns
        -------
        str

        """
        return _ogr.Geometry_GetGeometryName(self, *args)

    def Length(self, *args):
        r"""
        Length(self) -> double

        Compute geometry length.

        The returned length is a 2D Cartesian (planar) area in units of the
        spatial reference system in use, so potentially 'degrees' for a
        geometry expressed in a geographic SRS.

        For more details: :cpp:func:`OGR_G_Length`

        Returns
        -------
        float
            the length of the geometry in units of the spatial reference
            system in use, or 0.0 for unsupported geometry types.


        """
        return _ogr.Geometry_Length(self, *args)

    def Area(self, *args):
        r"""
        Area(self) -> double

        Compute geometry area.

        The returned area is a 2D Cartesian (planar) area in square units of the
        spatial reference system in use, so potentially 'square degrees' for a
        geometry expressed in a geographic SRS.

        For more details: :cpp:func:`OGR_G_Area`

        Returns
        -------
        float
            the area of the geometry in square units of the spatial reference
            system in use, or 0.0 for unsupported geometry types.


        """
        return _ogr.Geometry_Area(self, *args)

    def GeodesicLength(self, *args):
        r"""
        GeodesicLength(self) -> double

        Compute geometry length, considered as a curve on the underlying
        ellipsoid of the SRS attached to the geometry.

        For more details: :cpp:func:`OGR_G_GeodesicLength`

        Returns
        -------
        float
            the area in meters, or a negative value for unsupported geometry types.


        """
        return _ogr.Geometry_GeodesicLength(self, *args)

    def GeodesicArea(self, *args):
        r"""
        GeodesicArea(self) -> double

        Compute geometry area, considered as a surface on the underlying
        ellipsoid of the SRS attached to the geometry.

        For more details: :cpp:func:`OGR_G_GeodesicArea`

        Returns
        -------
        float
            the area in square meters, or a negative value for unsupported geometry types.


        """
        return _ogr.Geometry_GeodesicArea(self, *args)

    def IsClockwise(self, *args):
        r"""IsClockwise(self) -> bool"""
        return _ogr.Geometry_IsClockwise(self, *args)

    def GetArea(self, *args):
        r"""GetArea(self) -> double"""
        return _ogr.Geometry_GetArea(self, *args)

    def GetPointCount(self, *args):
        r"""GetPointCount(self) -> int"""
        return _ogr.Geometry_GetPointCount(self, *args)

    def GetPoints(self, *args, **kwargs):
        r"""GetPoints(self, nCoordDimension=0)"""
        return _ogr.Geometry_GetPoints(self, *args, **kwargs)

    def GetX(self, *args, **kwargs):
        r"""GetX(self, point=0) -> double"""
        return _ogr.Geometry_GetX(self, *args, **kwargs)

    def GetY(self, *args, **kwargs):
        r"""GetY(self, point=0) -> double"""
        return _ogr.Geometry_GetY(self, *args, **kwargs)

    def GetZ(self, *args, **kwargs):
        r"""GetZ(self, point=0) -> double"""
        return _ogr.Geometry_GetZ(self, *args, **kwargs)

    def GetM(self, *args, **kwargs):
        r"""GetM(self, point=0) -> double"""
        return _ogr.Geometry_GetM(self, *args, **kwargs)

    def GetPoint(self, *args):
        r"""GetPoint(self, iPoint=0)"""
        return _ogr.Geometry_GetPoint(self, *args)

    def GetPointZM(self, *args):
        r"""GetPointZM(self, iPoint=0)"""
        return _ogr.Geometry_GetPointZM(self, *args)

    def GetPoint_2D(self, *args):
        r"""GetPoint_2D(self, iPoint=0)"""
        return _ogr.Geometry_GetPoint_2D(self, *args)

    def GetGeometryCount(self, *args):
        r"""GetGeometryCount(self) -> int"""
        return _ogr.Geometry_GetGeometryCount(self, *args)

    def SetPoint(self, *args, **kwargs):
        r"""SetPoint(self, point, x, y, z=0) -> OGRErr"""
        return _ogr.Geometry_SetPoint(self, *args, **kwargs)

    def SetPointM(self, *args, **kwargs):
        r"""SetPointM(self, point, x, y, m) -> OGRErr"""
        return _ogr.Geometry_SetPointM(self, *args, **kwargs)

    def SetPointZM(self, *args, **kwargs):
        r"""SetPointZM(self, point, x, y, z, m) -> OGRErr"""
        return _ogr.Geometry_SetPointZM(self, *args, **kwargs)

    def SetPoint_2D(self, *args, **kwargs):
        r"""SetPoint_2D(self, point, x, y) -> OGRErr"""
        return _ogr.Geometry_SetPoint_2D(self, *args, **kwargs)

    def SwapXY(self, *args):
        r"""
        SwapXY(self)

        Swap x and y coordinates.

        For more details: :cpp:func:`OGR_G_SwapXY`

        .. versionadded:: 2.3.0


        """
        return _ogr.Geometry_SwapXY(self, *args)

    def GetGeometryRef(self, *args):
        r"""GetGeometryRef(self, geom) -> Geometry"""
        val = _ogr.Geometry_GetGeometryRef(self, *args)

        if val is not None:
            val._parent_geom = self


        return val


    def Simplify(self, *args):
        r"""
        Simplify(self, tolerance) -> Geometry

        Compute a simplified geometry.

        For more details: :cpp:func:`OGR_G_Simplify`

        Parameters
        ----------
        tolerance : float
            The distance tolerance for the simplification.

        Returns
        -------
        Geometry
            The simplified geometry or None if an error occurs.

        """
        return _ogr.Geometry_Simplify(self, *args)

    def SimplifyPreserveTopology(self, *args):
        r"""
        SimplifyPreserveTopology(self, tolerance) -> Geometry

        Simplify the geometry while preserving topology.

        For more details: :cpp:func:`OGR_G_SimplifyPreserveTopology`

        Parameters
        ----------
        tolerance : float
            The distance tolerance for the simplification.

        Returns
        -------
        Geometry
            The simplified geometry or None if an error occurs.

        """
        return _ogr.Geometry_SimplifyPreserveTopology(self, *args)

    def DelaunayTriangulation(self, *args, **kwargs):
        r"""
        DelaunayTriangulation(self, dfTolerance=0.0, bOnlyEdges=FALSE) -> Geometry

        Return a Delaunay triangulation of the vertices of the geometry.

        For more details: :cpp:func:`OGR_G_DelaunayTriangulation`

        .. versionadded:: 2.1

        Parameters
        ----------
        dfTolerance : float
            optional snapping tolerance to use for improved robustness
        bOnlyEdges : bool
            If True, will return a MULTILINESTRING, otherwise it will
            return a GEOMETRYCOLLECTION containing triangular POLYGONs.

        Returns
        -------
        Geometry
            The geometry resulting from the Delaunay triangulation or None if an
            error occurs.

        """
        return _ogr.Geometry_DelaunayTriangulation(self, *args, **kwargs)

    def ConstrainedDelaunayTriangulation(self, *args):
        r"""
        ConstrainedDelaunayTriangulation(self) -> Geometry

        Return a constrained Delaunay triangulation of the vertices of the given
        polygon(s). For non-polygonal inputs, silently returns an empty geometry
        collection.

        For more details: :cpp:func:`OGR_G_ConstrainedDelaunayTriangulation`

        .. versionadded:: 3.12

        Returns
        -------
        Geometry
            The geometry collection resulting from the constrained Delaunay
            triangulation or None if an error occurs.

        """
        return _ogr.Geometry_ConstrainedDelaunayTriangulation(self, *args)

    def Polygonize(self, *args):
        r"""
        Polygonize(self) -> Geometry

        Polygonizes a set of sparse edges.

        For more details: :cpp:func:`OGR_G_Polygonize`

        Returns
        -------
        Geometry
            A new geometry or None on failure.

        """
        return _ogr.Geometry_Polygonize(self, *args)

    def BuildArea(self, *args):
        r"""
        BuildArea(self) -> Geometry

        Polygonize a linework assuming inner polygons are holes.

        For more details: :cpp:func:`OGR_G_BuildArea`

        Returns
        -------
        Geometry
            A new geometry or None on failure.

        """
        return _ogr.Geometry_BuildArea(self, *args)

    def Boundary(self, *args):
        r"""
        Boundary(self) -> Geometry

        Compute boundary.

        For more details: :cpp:func:`OGR_G_Boundary`

        Returns
        -------
        Geometry
            A new geometry or None on failure.

        """
        return _ogr.Geometry_Boundary(self, *args)

    def GetBoundary(self, *args):
        r"""
        GetBoundary(self) -> Geometry

        Compute boundary (deprecated)

        For more details: :cpp:func:`OGR_G_GetBoundary`

        ..warning:: Deprecated

        See: :cpp:func:`OGR_G_Boundary`

        """
        return _ogr.Geometry_GetBoundary(self, *args)

    def ConvexHull(self, *args):
        r"""
        ConvexHull(self) -> Geometry

        Compute convex hull.

        For more details: :cpp:func:`OGR_G_ConvexHull`

        Returns
        -------
        Geometry
            a handle to A newly allocated geometry now owned by the caller, or
            None on failure.

        """
        return _ogr.Geometry_ConvexHull(self, *args)

    def ConcaveHull(self, *args):
        r"""
        ConcaveHull(self, ratio, allowHoles) -> Geometry

        Compute the concave hull of a geometry.

        For more details: :cpp:func:`OGR_G_ConcaveHull`

        See also :py:meth:`ConcaveHullOfPolygons`

        Parameters
        ----------
        ratio : float
            Ratio of the area of the convex hull and the concave hull.
        allowHoles : bool
            Whether the concave hull is allowed to contain holes.

        Returns
        -------
        Geometry
            The concave hull geometry or None if an error occurs.

        """
        return _ogr.Geometry_ConcaveHull(self, *args)

    def ConcaveHullOfPolygons(self, *args):
        r"""
        ConcaveHullOfPolygons(self, lengthRatio, isTight, allowHoles) -> Geometry

        Compute the concave hull of a set of polygons, respecting the polygons as constraints.

        For more details: :cpp:func:`OGR_G_ConcaveHullOfPolygons`

        Parameters
        ----------
        lengthRatio : float
            Specifies the Maximum Edge Length as a fraction of the
            difference between the longest and shortest edge lengths
            between the polygons.
            This normalizes the Maximum Edge Length to be scale-free.
            A value of 1 produces the convex hull; a value of 0 produces
            the original polygons.
        isTight : bool
            Whether the hull must follow the outer boundaries of the input polygons.
        allowHoles : bool
            Whether the concave hull is allowed to contain holes.

        Returns
        -------
        Geometry
            The concave hull geometry or None if an error occurs.

        """
        return _ogr.Geometry_ConcaveHullOfPolygons(self, *args)

    def MakeValid(self, *args):
        r"""
        MakeValid(self, options=None) -> Geometry

        Attempts to make an invalid geometry valid without losing vertices.

        For more details: :cpp:func:`OGR_G_MakeValidEx`

        .. versionadded:: 3.0
        .. versionadded:: 3.4 options

        Parameters
        ----------
        options : list[str], optional
            papszOptions to be passed in. For example: ["METHOD=STRUCTURE"].

        Returns
        -------
        Geometry
            A newly allocated geometry now owned by the caller, or None on
            failure.

        """
        return _ogr.Geometry_MakeValid(self, *args)

    def SetPrecision(self, *args):
        r"""SetPrecision(self, gridSize, flags=0) -> Geometry"""
        return _ogr.Geometry_SetPrecision(self, *args)

    def Normalize(self, *args):
        r"""
        Normalize(self) -> Geometry

        Attempts to bring geometry into normalized/canonical form.

        For more details: :cpp:func:`OGR_G_Normalize`

        .. versionadded:: 3.3

        Returns
        -------
        Geometry
            A newly allocated geometry now owned by the caller, or None on
            failure.

        """
        return _ogr.Geometry_Normalize(self, *args)

    def RemoveLowerDimensionSubGeoms(self, *args):
        r"""RemoveLowerDimensionSubGeoms(self) -> Geometry"""
        return _ogr.Geometry_RemoveLowerDimensionSubGeoms(self, *args)

    def Buffer(self, *args):
        r"""
        Buffer(self, distance, quadsecs=30) -> Geometry
        Buffer(self, distance, options) -> Geometry

        Compute buffer of geometry.

        For more details: :cpp:func:`OGR_G_Buffer`

        Parameters
        ----------
        distance : float
            The buffer distance to be applied. Should be expressed into
            the same unit as the coordinates of the geometry.
        quadsecs : int, default=30
            The number of segments used to approximate a 90 degree
            (quadrant) of curvature.
        options : list or dict, optional
            An optional list of options to control the buffer output.
            See :cpp:func:`OGR_G_BufferEx`.

        Returns
        -------
        Geometry
            The newly created geometry or None if an error occurs.

        """
        return _ogr.Geometry_Buffer(self, *args)

    def Intersection(self, *args):
        r"""
        Intersection(self, other) -> Geometry

        Compute intersection.

        For more details: :cpp:func:`OGR_G_Intersection`

        Parameters
        ----------
        other : Geometry
            The other geometry.

        Returns
        -------
        Geometry
            A new geometry representing the intersection or None if there is no
            intersection or an error occurs.

        """
        return _ogr.Geometry_Intersection(self, *args)

    def Union(self, *args):
        r"""
        Union(self, other) -> Geometry

        Compute union.

        For more details: :cpp:func:`OGR_G_Union`

        Parameters
        ----------
        other : Geometry
            The other geometry.

        Returns
        -------
        Geometry
            A new geometry representing the union or None if an error occurs.

        """
        return _ogr.Geometry_Union(self, *args)

    def UnionCascaded(self, *args):
        r"""
        UnionCascaded(self) -> Geometry

        Compute union using cascading.

        For more details: :cpp:func:`OGR_G_UnionCascaded`

        Returns
        -------
        Geometry
            A new geometry representing the union or None if an error occurs.

        """
        return _ogr.Geometry_UnionCascaded(self, *args)

    def UnaryUnion(self, *args):
        r"""UnaryUnion(self) -> Geometry"""
        return _ogr.Geometry_UnaryUnion(self, *args)

    def Difference(self, *args):
        r"""
        Difference(self, other) -> Geometry

        Compute difference.

        For more details: :cpp:func:`OGR_G_Difference`

        Parameters
        ----------
        other : Geometry
            The other geometry.

        Returns
        -------
        Geometry
            A new geometry representing the difference or None if the difference
            is empty or an error occurs.

        """
        return _ogr.Geometry_Difference(self, *args)

    def SymDifference(self, *args):
        r"""
        SymDifference(self, other) -> Geometry

        Compute symmetric difference.

        For more details: :cpp:func:`OGR_G_SymDifference`

        Parameters
        ----------
        other : Geometry
            the other geometry.

        Returns
        -------
        Geometry
            A new geometry representing the symmetric difference or None if the
            difference is empty or an error occurs.

        """
        return _ogr.Geometry_SymDifference(self, *args)

    def SymmetricDifference(self, *args):
        r"""
        SymmetricDifference(self, other) -> Geometry

        Compute symmetric difference (deprecated)

        For more details: :cpp:func:`OGR_G_SymmetricDifference`

        .. warning:: Deprecated


        """
        return _ogr.Geometry_SymmetricDifference(self, *args)

    def Distance(self, *args):
        r"""
        Distance(self, other) -> double

        Compute distance between two geometries.

        For more details: :cpp:func:`OGR_G_Distance`

        Parameters
        ----------
        other : Geometry
            The other geometry to compare against.

        Returns
        -------
        float
            The distance between the geometries or -1 if an error occurs.

        """
        return _ogr.Geometry_Distance(self, *args)

    def Distance3D(self, *args):
        r"""
        Distance3D(self, other) -> double

        Returns the 3D distance between two geometries.

        For more details: :cpp:func:`OGR_G_Distance3D`

        .. versionadded:: 2.2

        Parameters
        ----------
        other : Geometry
            The other geometry to compare against.

        Returns
        -------
        float
            The distance between the geometries or -1 if an error occurs.

        """
        return _ogr.Geometry_Distance3D(self, *args)

    def Empty(self, *args):
        r"""
        Empty(self)

        Clear geometry information.

        For more details: :cpp:func:`OGR_G_Empty`

        """
        return _ogr.Geometry_Empty(self, *args)

    def IsEmpty(self, *args):
        r"""
        IsEmpty(self) -> bool

        Test if the geometry is empty.

        For more details: :cpp:func:`OGR_G_IsEmpty`

        Returns
        -------
        int
            True if the geometry has no points, otherwise False.

        """
        return _ogr.Geometry_IsEmpty(self, *args)

    def IsValid(self, *args):
        r"""
        IsValid(self) -> bool

        Test if the geometry is valid.

        For more details: :cpp:func:`OGR_G_IsValid`

        Returns
        -------
        int
            True if the geometry is invalid, otherwise False.

        """
        return _ogr.Geometry_IsValid(self, *args)

    def GetInvalidityReason(self, *args):
        r"""
        GetInvalidityReason(self) -> retStringAndCPLFree *

        Test if the geometry is valid and, if not, return the invalidity reason.

        For more details: :cpp:func:`OGR_G_GetInvalidityReason`

        Returns
        -------
        str
            a string with the invalidity reason, or None if it is valid

        """
        return _ogr.Geometry_GetInvalidityReason(self, *args)

    def IsSimple(self, *args):
        r"""
        IsSimple(self) -> bool

        Returns True if the geometry is simple.

        For more details: :cpp:func:`OGR_G_IsSimple`

        Returns
        -------
        int
            True if object is simple, otherwise False.

        """
        return _ogr.Geometry_IsSimple(self, *args)

    def IsRing(self, *args):
        r"""
        IsRing(self) -> bool

        Test if the geometry is a ring.

        For more details: :cpp:func:`OGR_G_IsRing`

        Returns
        -------
        int
            True if the coordinates of the geometry form a ring, by checking length
            and closure (self-intersection is not checked), otherwise False.

        """
        return _ogr.Geometry_IsRing(self, *args)

    def Intersects(self, *args):
        r"""
        Intersects(self, other) -> bool

        Determines whether two geometries intersect.

        For more details: :cpp:func:`OGR_G_Intersects`

        Parameters
        ----------
        other : Geometry
            The other geometry to test against.

        Returns
        -------
        int
            True if the geometries intersect, otherwise False.

        """
        return _ogr.Geometry_Intersects(self, *args)

    def Intersect(self, *args):
        r"""Intersect(self, other) -> bool"""
        return _ogr.Geometry_Intersect(self, *args)

    def Equals(self, *args):
        r"""
        Equals(self, other) -> bool

        Returns True if two geometries are equivalent.

        For more details: :cpp:func:`OGR_G_Equals`

        Parameters
        ----------
        other : Geometry
            The other geometry to test against.

        Returns
        -------
        int
            True if equivalent or False otherwise.

        """
        return _ogr.Geometry_Equals(self, *args)

    def Equal(self, *args):
        r"""Equal(self, other) -> bool"""
        return _ogr.Geometry_Equal(self, *args)

    def Disjoint(self, *args):
        r"""
        Disjoint(self, other) -> bool

        Test for disjointness.

        For more details: :cpp:func:`OGR_G_Disjoint`

        Parameters
        ----------
        other : Geometry
            The other geometry to compare.

        Returns
        -------
        int
            True if they are disjoint, otherwise False.

        """
        return _ogr.Geometry_Disjoint(self, *args)

    def Touches(self, *args):
        r"""
        Touches(self, other) -> bool

        Test for touching.

        For more details: :cpp:func:`OGR_G_Touches`

        Parameters
        ----------
        other : Geometry
            the other geometry to compare.

        Returns
        -------
        int
            True if they are touching, otherwise False.

        """
        return _ogr.Geometry_Touches(self, *args)

    def Crosses(self, *args):
        r"""
        Crosses(self, other) -> bool

        Test for crossing.

        For more details: :cpp:func:`OGR_G_Crosses`

        Parameters
        ----------
        other : Geometry
            the other geometry to compare.

        Returns
        -------
        int
            True if they are crossing, otherwise False.

        """
        return _ogr.Geometry_Crosses(self, *args)

    def Within(self, *args):
        r"""
        Within(self, other) -> bool

        Test for containment.

        For more details: :cpp:func:`OGR_G_Within`

        Parameters
        ----------
        other : Geometry
            the other geometry to compare.

        Returns
        -------
        int
            True if this is within other, otherwise False.

        """
        return _ogr.Geometry_Within(self, *args)

    def Contains(self, *args):
        r"""
        Contains(self, other) -> bool

        Test for containment.

        For more details: :cpp:func:`OGR_G_Contains`

        Parameters
        ----------
        other : Geometry
            the other geometry to compare.

        Returns
        -------
        int
            True if this contains the other geometry, otherwise False.

        """
        return _ogr.Geometry_Contains(self, *args)

    def Overlaps(self, *args):
        r"""
        Overlaps(self, other) -> bool

        Test for overlap.

        For more details: :cpp:func:`OGR_G_Overlaps`

        Parameters
        ----------
        other : Geometry
            the other geometry to compare.

        Returns
        -------
        int
            True if they are overlapping, otherwise False.

        """
        return _ogr.Geometry_Overlaps(self, *args)

    def TransformTo(self, *args):
        r"""
        TransformTo(self, reference) -> OGRErr

        Transform geometry to new spatial reference system.

        For more details: :cpp:func:`OGR_G_TransformTo`

        Parameters
        ----------
        reference : SpatialReference
           The spatial reference system to apply.

        Returns
        -------
        int
            :py:const:`osgeo.ogr.OGRERR_NONE` on success, or an error code.

        """
        return _ogr.Geometry_TransformTo(self, *args)

    def GetSpatialReference(self, *args):
        r"""
        GetSpatialReference(self) -> SpatialReference

        For more details: :cpp:func:`OGR_G_GetSpatialReference`

        Returns spatial reference system for geometry.

        Returns
        -------
        SpatialReference

        """
        return _ogr.Geometry_GetSpatialReference(self, *args)

    def AssignSpatialReference(self, *args):
        r"""
        AssignSpatialReference(self, reference)

        Assign spatial reference to this object.

        For more details: :cpp:func:`OGR_G_AssignSpatialReference`

        Parameters
        ----------
        reference : SpatialReference
            The new spatial reference system to apply.

        """
        return _ogr.Geometry_AssignSpatialReference(self, *args)

    def CloseRings(self, *args):
        r"""
        CloseRings(self)

        Force rings to be closed.

        For more details: :cpp:func:`OGR_G_CloseRings`

        """
        return _ogr.Geometry_CloseRings(self, *args)

    def FlattenTo2D(self, *args):
        r"""
        FlattenTo2D(self)

        Convert geometry to strictly 2D.

        For more details: :cpp:func:`OGR_G_FlattenTo2D`

        """
        return _ogr.Geometry_FlattenTo2D(self, *args)

    def Segmentize(self, *args):
        r"""
        Segmentize(self, dfMaxLength)

        Modify the geometry such it has no segment longer then the given
        distance.

        For more details: :cpp:func:`OGR_G_Segmentize`

        Parameters
        ----------
        dfMaxLength : float
            the maximum distance between 2 points after segmentization

        """
        return _ogr.Geometry_Segmentize(self, *args)

    def GetEnvelope(self, *args):
        r"""
        GetEnvelope(self)

        Computes and returns the bounding envelope for this geometry in the
        passed psEnvelope structure.

        For more details: :cpp:func:`OGR_G_GetEnvelope`

        .. warning:: Check the return order of the bounds.

        Returns
        -------
        tuple of float
            (minx, maxx, miny, maxy)

        """
        return _ogr.Geometry_GetEnvelope(self, *args)

    def GetEnvelope3D(self, *args):
        r"""
        GetEnvelope3D(self)

        Computes and returns the bounding envelope (3D) for this geometry in
        the passed psEnvelope structure.

        For more details: :cpp:func:`OGR_G_GetEnvelope3D`

        .. warning:: Check the return order of the bounds.

        Returns
        -------
        tuple of float
            (minx, maxx, miny, maxy, minz, maxz)

        """
        return _ogr.Geometry_GetEnvelope3D(self, *args)

    def Centroid(self, *args):
        r"""
        Centroid(self) -> Geometry

        Compute the geometry centroid.

        For more details: :cpp:func:`OGR_G_Centroid`

        Returns
        -------
        Geometry

        """
        return _ogr.Geometry_Centroid(self, *args)

    def PointOnSurface(self, *args):
        r"""
        PointOnSurface(self) -> Geometry

        Returns a point guaranteed to lie on the surface.

        For more details: :cpp:func:`OGR_G_PointOnSurface`

        Returns
        -------
        Geometry
            A point guaranteed to lie on the surface or None if an error occurred.

        """
        return _ogr.Geometry_PointOnSurface(self, *args)

    def WkbSize(self, *args):
        r"""
        WkbSize(self) -> size_t

        Returns size of related binary representation.

        For more details: :cpp:func:`OGR_G_WkbSize`

        Returns
        -------
        int

        """
        return _ogr.Geometry_WkbSize(self, *args)

    def GetCoordinateDimension(self, *args):
        r"""
        GetCoordinateDimension(self) -> int

        Get the dimension of the coordinates in this geometry.

        For more details: :cpp:func:`OGR_G_GetCoordinateDimension`

        .. warning:: Deprecated. Use :py:func:`CoordinateDimension`,
            :py:func:`Is3D`, or :py:func:`IsMeasured`.

        Returns
        -------
        int
            This will return 2 or 3.

        """
        return _ogr.Geometry_GetCoordinateDimension(self, *args)

    def CoordinateDimension(self, *args):
        r"""
        CoordinateDimension(self) -> int

        Get the dimension of the coordinates in this geometry.

        For more details: :cpp:func:`OGR_G_CoordinateDimension`

        .. versionadded:: 2.1

        Returns
        -------
        int
            This will return 2 for XY, 3 for XYZ and XYM, and 4 for XYZM data.

        """
        return _ogr.Geometry_CoordinateDimension(self, *args)

    def Is3D(self, *args):
        r"""
        Is3D(self) -> int

        See whether this geometry has Z coordinates.

        For more details: :cpp:func:`OGR_G_Is3D`

        .. versionadded:: 2.1

        Returns
        -------
        int
            True if the geometry has Z coordinates.

        """
        return _ogr.Geometry_Is3D(self, *args)

    def IsMeasured(self, *args):
        r"""
        IsMeasured(self) -> int

        See whether this geometry is measured.

        For more details: :cpp:func:`OGR_G_IsMeasured`

        .. versionadded:: 2.1

        Returns
        -------
        int
            True if the geometry has M coordinates.

        """
        return _ogr.Geometry_IsMeasured(self, *args)

    def SetCoordinateDimension(self, *args):
        r"""
        SetCoordinateDimension(self, dimension)

        Set the coordinate dimension.

        For more details: :cpp:func:`OGR_G_SetCoordinateDimension`

        .. warning:: Deprecated. Use :py:func:`Set3D` or :py:func:`SetMeasured`.

        Parameters
        ----------
        dimension : int
            New coordinate dimension value, either 2 or 3.

        """
        return _ogr.Geometry_SetCoordinateDimension(self, *args)

    def Set3D(self, *args):
        r"""
        Set3D(self, b3D)

        Add or remove the Z coordinate dimension.

        For more details: :cpp:func:`OGR_G_Set3D`

        .. versionadded:: 2.1

        Parameters
        ----------
        bIs3D : bool
            Should the geometry have a Z dimension, either True or False.

        """
        return _ogr.Geometry_Set3D(self, *args)

    def SetMeasured(self, *args):
        r"""
        SetMeasured(self, bMeasured)

        Add or remove the M coordinate dimension.

        For more details: :cpp:func:`OGR_G_SetMeasured`

        .. versionadded:: 2.1

        Parameters
        ----------
        bIsMeasured : bool
            Should the geometry have a M dimension, either True or False.

        """
        return _ogr.Geometry_SetMeasured(self, *args)

    def GetDimension(self, *args):
        r"""
        GetDimension(self) -> int

        Get the dimension of this geometry.

        For more details: :cpp:func:`OGR_G_GetDimension`

        Returns
        -------
        int
            0 for points, 1 for lines, and 2 for surfaces.

        """
        return _ogr.Geometry_GetDimension(self, *args)

    def HasCurveGeometry(self, *args):
        r"""HasCurveGeometry(self, bLookForCircular=FALSE) -> int"""
        return _ogr.Geometry_HasCurveGeometry(self, *args)

    def GetLinearGeometry(self, *args, **kwargs):
        r"""GetLinearGeometry(self, dfMaxAngleStepSizeDegrees=0.0, options=None) -> Geometry"""
        return _ogr.Geometry_GetLinearGeometry(self, *args, **kwargs)

    def GetCurveGeometry(self, *args, **kwargs):
        r"""GetCurveGeometry(self, options=None) -> Geometry"""
        return _ogr.Geometry_GetCurveGeometry(self, *args, **kwargs)

    def Value(self, *args):
        r"""Value(self, dfDistance) -> Geometry"""
        return _ogr.Geometry_Value(self, *args)

    def Transform(self, *args):
        r"""
        Transform(self, trans) -> OGRErr
        Transform(self, transformer) -> Geometry

        Apply a coordinate transformation to the geometry.

        The behavior of the function depends on the type of the provided argument.

        If a :py:class:`osr.CoordinateTransformation` is provided, the geometry will be
        transformed in-place, or an error code returned.

        If a :py:class:`GeomTransformer` is provided, the geometry will remain unmodified,
        and a transformed geometry will be returned.

        See :cpp:func:`OGR_G_Transform` and :cpp:func:`OGR_GeomTransformer_Transform`.

        Parameters
        ----------
        trans : CoordinateTransformation or GeomTransformer
            The transformation to apply.

        Returns
        -------
        Geometry or int
            The transformed geometry if ``trans`` is a :py:class:`GeomTransformer`, or an error code if ``trans`` is a :py:class:`osr.CoordinateTransformation`.

        Examples
        --------
        >>> # use a CoordinateTransformation to transform a geometry in-place
        >>> wgs84 = osr.SpatialReference(epsg=4326)
        >>> wgs84.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
        >>> albers = osr.SpatialReference(epsg=9473)
        >>> xform = osr.CoordinateTransformation(wgs84, albers)
        >>> vt = ogr.Geometry(ogr.wkbPoint)
        >>> vt.AddPoint_2D(145.195, -37.836)
        0
        >>> vt.Transform(xform)
        0
        >>> vt.GetPoint_2D()
        (1166651.434325419, -4196676.777645577)

        >>> # use a GeomTransformer and return a transformed geometry
        >>> polar_stereo = osr.SpatialReference(epsg=3413)
        >>> rect = ogr.CreateGeometryFromWkt('POLYGON((-3106890 145265,-3106890 2474514,-672244 2474514,-672244 145265,-3106890 145265))')
        >>> xform = osr.CoordinateTransformation(polar_stereo, wgs84)
        >>> xformer = ogr.GeomTransformer(xform, {'WRAPDATELINE=TRUE':True, 'DATELINEOFFSET': 180})
        >>> rect_transformed = rect.Transform(xformer)
        >>> rect_transformed.ExportToWkt()
        'MULTIPOLYGON (((150.198564480925 66.6449112280509,180.0 74.7428375082776,180.0 56.6419000989319,150.198564480925 66.6449112280509)),((-173.535923883218 54.4721983980102,-180 56.6419000989319,-180 74.7428375082776,-147.193543096797 83.6573164550226,-137.676958092793 61.8457340532701,-173.535923883218 54.4721983980102)))' # no-check

        """
        return _ogr.Geometry_Transform(self, *args)

    def CreatePreparedGeometry(self, *args):
        r"""CreatePreparedGeometry(self) -> PreparedGeometry"""
        return _ogr.Geometry_CreatePreparedGeometry(self, *args)

    def Destroy(self):
      self.__swig_destroy__(self)
      self.thisown = 0

    def __str__(self):
      return self.ExportToIsoWkt()

    def __copy__(self):
      return self.Clone()

    def __deepcopy__(self, memo):
      g = self.Clone()
      srs = self.GetSpatialReference()
      if srs:
          g.AssignSpatialReference(srs.Clone())
      return g

    def __reduce__(self):
      return (self.__class__, (), self.ExportToWkb())

    def __setstate__(self, state):
        result = CreateGeometryFromWkb(state)
        self.this = result.this

    def __iter__(self):
        for i in range(self.GetGeometryCount()):
            yield self.GetGeometryRef(i)


# Register Geometry in _ogr:
_ogr.Geometry_swigregister(Geometry)
class PreparedGeometry(object):
    r"""Proxy of C++ OGRPreparedGeometryShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_PreparedGeometry

    def Intersects(self, *args):
        r"""Intersects(self, otherGeom) -> bool"""
        return _ogr.PreparedGeometry_Intersects(self, *args)

    def Contains(self, *args):
        r"""Contains(self, otherGeom) -> bool"""
        return _ogr.PreparedGeometry_Contains(self, *args)

# Register PreparedGeometry in _ogr:
_ogr.PreparedGeometry_swigregister(PreparedGeometry)
class GeomTransformer(object):
    r"""Proxy of C++ OGRGeomTransformerShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(self, ct, options=None) -> GeomTransformer"""
        _ogr.GeomTransformer_swiginit(self, _ogr.new_GeomTransformer(*args))
    __swig_destroy__ = _ogr.delete_GeomTransformer

    def Transform(self, *args):
        r"""Transform(self, src_geom) -> Geometry"""
        return _ogr.GeomTransformer_Transform(self, *args)

# Register GeomTransformer in _ogr:
_ogr.GeomTransformer_swigregister(GeomTransformer)
class FieldDomain(object):
    r"""


    Python proxy of an :cpp:class:`OGRFieldDomain`.

    Created using one of:

    - :py:func:`CreateCodedFieldDomain`
    - :py:func:`CreateGlobFieldDomain`
    - :py:func:`CreateRangeFieldDomain`

    """

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_FieldDomain

    def GetName(self, *args):
        r"""
        GetName(self) -> char const *


        Get the name of the field domain.

        See :cpp:func:`OGRFieldDomain::GetName`.

        .. versionadded:: 3.3

        Returns
        -------
        str
            the field domain name.

        """
        return _ogr.FieldDomain_GetName(self, *args)

    def GetDescription(self, *args):
        r"""
        GetDescription(self) -> char const *


        Get the description of the field domain.

        See :cpp:func:`OGRFieldDomain::GetDescription`.

        .. versionadded:: 3.3

        Returns
        -------
        str
            the field domain description (might be empty string).

        """
        return _ogr.FieldDomain_GetDescription(self, *args)

    def GetFieldType(self, *args):
        r"""
        GetFieldType(self) -> OGRFieldType


        Get the field type of the field domain.

        See :cpp:func:`OGRFieldDomain::GetFieldType`.

        .. versionadded:: 3.3

        Returns
        -------
        int
            the field type of the field domain.

        """
        return _ogr.FieldDomain_GetFieldType(self, *args)

    def GetFieldSubType(self, *args):
        r"""
        GetFieldSubType(self) -> OGRFieldSubType


        Get the field subtype of the field domain.

        See :cpp:func:`OGRFieldDomain::GetFieldSubType`.

        .. versionadded:: 3.3

        Returns
        -------
        int
            the field subtype of the field domain.

        """
        return _ogr.FieldDomain_GetFieldSubType(self, *args)

    def GetDomainType(self, *args):
        r"""
        GetDomainType(self) -> OGRFieldDomainType


        Get the type of the field domain.

        See :cpp:func:`OGRFieldDomain::GetDomainType`.

        .. versionadded:: 3.3

        Returns
        -------
        int
            the type of the field domain.

        Examples
        --------
        >>> d = ogr.CreateCodedFieldDomain('my_code', None, ogr.OFTInteger, ogr.OFSTNone, { 1 : 'owned', 2 : 'leased' })
        >>> d.GetDomainType() == ogr.OFDT_CODED
        True


        """
        return _ogr.FieldDomain_GetDomainType(self, *args)

    def GetSplitPolicy(self, *args):
        r"""
        GetSplitPolicy(self) -> OGRFieldDomainSplitPolicy


        Get the split policy of the field domain.

        See :cpp:func:`OGRFieldDomain::GetSplitPolicy`.

        .. versionadded:: 3.3

        Returns
        -------
        int
            the split policy of the field domain (default = :py:const:`OFDSP_DEFAULT_VALUE`)

        """
        return _ogr.FieldDomain_GetSplitPolicy(self, *args)

    def SetSplitPolicy(self, *args):
        r"""
        SetSplitPolicy(self, policy)


        Set the split policy of the field domain.

        See :cpp:func:`OGRFieldDomain::SetSplitPolicy`.

        .. versionadded:: 3.3

        policy : int
            the split policy code of the field domain.

        """
        return _ogr.FieldDomain_SetSplitPolicy(self, *args)

    def GetMergePolicy(self, *args):
        r"""
        GetMergePolicy(self) -> OGRFieldDomainMergePolicy


        Get the merge policy of the field domain.

        See :cpp:func:`OGRFieldDomain::GetMergePolicy`.

        .. versionadded:: 3.3

        Returns
        -------
        int
            the merge policy of the field domain (default = :py:const:`OFDMP_DEFAULT_VALUE`)

        """
        return _ogr.FieldDomain_GetMergePolicy(self, *args)

    def SetMergePolicy(self, *args):
        r"""
        SetMergePolicy(self, policy)


        Set the merge policy of the field domain.

        See :cpp:func:`OGRFieldDomain::SetMergePolicy`.

        .. versionadded:: 3.3

        Parameters
        ----------
        policy : int
            the merge policy code of the field domain.

        """
        return _ogr.FieldDomain_SetMergePolicy(self, *args)

    def GetEnumeration(self, *args):
        r"""
        GetEnumeration(self) -> OGRCodedValue const *


        Get the enumeration as a mapping of codes to values.

        See :cpp:func:`OGRCodedFieldDomain::GetEnumeration`.

        .. versionadded:: 3.3

        Returns
        -------
        dict

        Examples
        --------
        >>> d = ogr.CreateCodedFieldDomain('my_domain', None, ogr.OFTInteger, ogr.OFSTNone, { 1 : 'owned', 2 : 'leased' })
        >>> d.GetEnumeration()
        {'1': 'owned', '2': 'leased'}


        """
        return _ogr.FieldDomain_GetEnumeration(self, *args)

    def GetMinAsDouble(self, *args):
        r"""
        GetMinAsDouble(self) -> double


        Get the minimum value of a range domain.

        See :cpp:func:`OGRRangeFieldDomain::GetMin()`

        .. versionadded:: 3.3

        Returns
        -------
        float
            the minimum value of the range

        """
        return _ogr.FieldDomain_GetMinAsDouble(self, *args)

    def GetMinAsString(self, *args):
        r"""
        GetMinAsString(self) -> char const *


        Get the minimum value of a range domain.

        See :cpp:func:`OGRRangeFieldDomain::GetMin()`

        .. versionadded:: 3.3

        Returns
        -------
        str
            the minimum value of the range

        """
        return _ogr.FieldDomain_GetMinAsString(self, *args)

    def IsMinInclusive(self, *args):
        r"""IsMinInclusive(self) -> bool"""
        return _ogr.FieldDomain_IsMinInclusive(self, *args)

    def GetMaxAsDouble(self, *args):
        r"""
        GetMaxAsDouble(self) -> double


        Get the maximum value of a range domain.

        See :cpp:func:`OGRRangeFieldDomain::GetMax()`

        .. versionadded:: 3.3

        Returns
        -------
        float
            the maximum value of the range

        """
        return _ogr.FieldDomain_GetMaxAsDouble(self, *args)

    def GetMaxAsString(self, *args):
        r"""
        GetMaxAsString(self) -> char const *


        Get the maximum value of a range domain.

        See :cpp:func:`OGRRangeFieldDomain::GetMax()`

        .. versionadded:: 3.3

        Returns
        -------
        str
            the maximum value of the range

        """
        return _ogr.FieldDomain_GetMaxAsString(self, *args)

    def IsMaxInclusive(self, *args):
        r"""IsMaxInclusive(self) -> bool"""
        return _ogr.FieldDomain_IsMaxInclusive(self, *args)

    def GetGlob(self, *args):
        r"""
        GetGlob(self) -> char const *


        Get the glob expression.

        See :cpp:func:`OGRGlobFieldDomain::GetGlob`.

        .. versionadded:: 3.3

        Returns
        -------
        str
            the glob expression, or ``None`` in case of error

        """
        return _ogr.FieldDomain_GetGlob(self, *args)

# Register FieldDomain in _ogr:
_ogr.FieldDomain_swigregister(FieldDomain)

def CreateCodedFieldDomain(*args):
    r"""
    CreateCodedFieldDomain(name, description, type, subtype, enumeration) -> FieldDomain


    Creates a new coded field domain.

    See :cpp:func:`OGRCodedFieldDomain::OGRCodedFieldDomain`.

    .. versionadded:: 3.3

    Parameters
    ----------
    name : str
        Domain name. Should not be ``None``.
    description : str, optional
        Domain description (can be ``None``)
    type : int
        Field type.
    subtype : int
        Field subtype.
    enumeration : dict
        Enumeration as a dictionary of (code : value) pairs. Should not be ``None``.

    Returns
    -------
    FieldDomain

    """
    return _ogr.CreateCodedFieldDomain(*args)

def CreateRangeFieldDomain(*args):
    r"""
    CreateRangeFieldDomain(name, description, type, subtype, min, minIsInclusive, max, maxIsInclusive) -> FieldDomain

    Creates a new range field domain.

    See :cpp:func:`OGRRangeFieldDomain::OGRRangeFieldDomain`.

    .. versionadded:: 3.3

    Parameters
    ----------
    name : str
        Domain name. Should not be ``None``.
    description : str, optional
        Domain description (can be ``None``)
    type : int
        Field type. Generally numeric. Potentially :py:const:`OFTDateTime`.
    subtype : int
        Field subtype.
    min : float, optional
        Minimum value (can be ``None``).
    minIsInclusive : bool
        Whether the minimum value is included in the range.
    max : float, optional
        Maximum value (can be ``None``).
    maxIsInclusive : bool
        Whether the maximum value is included in the range.

    Returns
    -------
    FieldDomain

    """
    return _ogr.CreateRangeFieldDomain(*args)

def CreateRangeFieldDomainDateTime(*args):
    r"""CreateRangeFieldDomainDateTime(name, description, min, minIsInclusive, max, maxIsInclusive) -> FieldDomain"""
    return _ogr.CreateRangeFieldDomainDateTime(*args)

def CreateGlobFieldDomain(*args):
    r"""
    CreateGlobFieldDomain(name, description, type, subtype, glob) -> FieldDomain


    Creates a new glob field domain.

    See :cpp:func:`OGRGlobFieldDomain::OGRGlobFieldDomain`

    .. versionadded:: 3.3

    Parameters
    ----------
    name : str
        Domain name. Should not be ``None``.
    description : str, optional
        Domain description (can be ``None``)
    type : int
        Field type.
    subtype : int
        Field subtype.
    glob : str
        Glob expression. Should not be ``None``.

    Returns
    -------
    FieldDomain

    """
    return _ogr.CreateGlobFieldDomain(*args)
class GeomCoordinatePrecision(object):
    r"""Proxy of C++ OGRGeomCoordinatePrecisionShadow class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _ogr.delete_GeomCoordinatePrecision

    def Set(self, *args):
        r"""Set(self, xyResolution, zResolution, mResolution)"""
        return _ogr.GeomCoordinatePrecision_Set(self, *args)

    def SetFromMeter(self, *args):
        r"""SetFromMeter(self, srs, xyMeterResolution, zMeterResolution, mResolution)"""
        return _ogr.GeomCoordinatePrecision_SetFromMeter(self, *args)

    def GetXYResolution(self, *args):
        r"""GetXYResolution(self) -> double"""
        return _ogr.GeomCoordinatePrecision_GetXYResolution(self, *args)

    def GetZResolution(self, *args):
        r"""GetZResolution(self) -> double"""
        return _ogr.GeomCoordinatePrecision_GetZResolution(self, *args)

    def GetMResolution(self, *args):
        r"""GetMResolution(self) -> double"""
        return _ogr.GeomCoordinatePrecision_GetMResolution(self, *args)

    def GetFormats(self, *args):
        r"""GetFormats(self) -> char **"""
        return _ogr.GeomCoordinatePrecision_GetFormats(self, *args)

    def GetFormatSpecificOptions(self, *args):
        r"""GetFormatSpecificOptions(self, formatName) -> char **"""
        return _ogr.GeomCoordinatePrecision_GetFormatSpecificOptions(self, *args)

    def SetFormatSpecificOptions(self, *args):
        r"""SetFormatSpecificOptions(self, formatName, formatSpecificOptions)"""
        return _ogr.GeomCoordinatePrecision_SetFormatSpecificOptions(self, *args)

# Register GeomCoordinatePrecision in _ogr:
_ogr.GeomCoordinatePrecision_swigregister(GeomCoordinatePrecision)

def CreateGeomCoordinatePrecision(*args):
    r"""CreateGeomCoordinatePrecision() -> GeomCoordinatePrecision"""
    return _ogr.CreateGeomCoordinatePrecision(*args)

def GetDriverCount(*args):
    r"""GetDriverCount() -> int"""
    return _ogr.GetDriverCount(*args)

def GetOpenDSCount(*args):
    r"""GetOpenDSCount() -> int"""
    return _ogr.GetOpenDSCount(*args)

def SetGenerate_DB2_V72_BYTE_ORDER(*args):
    r"""SetGenerate_DB2_V72_BYTE_ORDER(bGenerate_DB2_V72_BYTE_ORDER) -> OGRErr"""
    return _ogr.SetGenerate_DB2_V72_BYTE_ORDER(*args)

def RegisterAll(*args):
    r"""RegisterAll()"""
    return _ogr.RegisterAll(*args)

def GeometryTypeToName(*args):
    r"""GeometryTypeToName(eType) -> char const *"""
    return _ogr.GeometryTypeToName(*args)

def GetFieldTypeName(*args):
    r"""
    GetFieldTypeName(type) -> char const *

    Fetch human readable name for a field type.

    See :cpp:func:`OGRFieldDefn::GetFieldTypeName`.

    Parameters
    ----------
    type : int
        the field type code to get name for

    Returns
    -------
    str
        the name

    Examples
    --------
    >>> ogr.GetFieldTypeName(0)
    'Integer'

    >>> ogr.GetFieldTypeName(ogr.OFTReal)
    'Real'

    """
    return _ogr.GetFieldTypeName(*args)

def GetFieldSubTypeName(*args):
    r"""
    GetFieldSubTypeName(type) -> char const *


    Fetch human readable name for a field subtype.

    See :cpp:func:`OGRFieldDefn::GetFieldSubTypeName`.

    Parameters
    ----------
    type : int
        the field subtype to get name for.

    Returns
    -------
    str
        the name.

    Examples
    --------
    >>> ogr.GetFieldSubTypeName(1)
    'Boolean'

    >>> ogr.GetFieldSubTypeName(ogr.OFSTInt16)
    'Int16'


    """
    return _ogr.GetFieldSubTypeName(*args)

def GT_Flatten(*args):
    r"""GT_Flatten(eType) -> OGRwkbGeometryType"""
    return _ogr.GT_Flatten(*args)

def GT_SetZ(*args):
    r"""GT_SetZ(eType) -> OGRwkbGeometryType"""
    return _ogr.GT_SetZ(*args)

def GT_SetM(*args):
    r"""GT_SetM(eType) -> OGRwkbGeometryType"""
    return _ogr.GT_SetM(*args)

def GT_SetModifier(*args):
    r"""GT_SetModifier(eType, bSetZ, bSetM=FALSE) -> OGRwkbGeometryType"""
    return _ogr.GT_SetModifier(*args)

def GT_HasZ(*args):
    r"""GT_HasZ(eType) -> int"""
    return _ogr.GT_HasZ(*args)

def GT_HasM(*args):
    r"""GT_HasM(eType) -> int"""
    return _ogr.GT_HasM(*args)

def GT_IsSubClassOf(*args):
    r"""GT_IsSubClassOf(eType, eSuperType) -> int"""
    return _ogr.GT_IsSubClassOf(*args)

def GT_IsCurve(*args):
    r"""GT_IsCurve(arg1) -> int"""
    return _ogr.GT_IsCurve(*args)

def GT_IsSurface(*args):
    r"""GT_IsSurface(arg1) -> int"""
    return _ogr.GT_IsSurface(*args)

def GT_IsNonLinear(*args):
    r"""GT_IsNonLinear(arg1) -> int"""
    return _ogr.GT_IsNonLinear(*args)

def GT_GetCollection(*args):
    r"""GT_GetCollection(eType) -> OGRwkbGeometryType"""
    return _ogr.GT_GetCollection(*args)

def GT_GetSingle(*args):
    r"""GT_GetSingle(eType) -> OGRwkbGeometryType"""
    return _ogr.GT_GetSingle(*args)

def GT_GetCurve(*args):
    r"""GT_GetCurve(eType) -> OGRwkbGeometryType"""
    return _ogr.GT_GetCurve(*args)

def GT_GetLinear(*args):
    r"""GT_GetLinear(eType) -> OGRwkbGeometryType"""
    return _ogr.GT_GetLinear(*args)

def SetNonLinearGeometriesEnabledFlag(*args):
    r"""SetNonLinearGeometriesEnabledFlag(bFlag)"""
    return _ogr.SetNonLinearGeometriesEnabledFlag(*args)

def GetNonLinearGeometriesEnabledFlag(*args):
    r"""GetNonLinearGeometriesEnabledFlag() -> int"""
    return _ogr.GetNonLinearGeometriesEnabledFlag(*args)

def GetOpenDS(*args):
    r"""GetOpenDS(ds_number) -> OGRDataSourceShadow *"""
    return _ogr.GetOpenDS(*args)

def Open(*args, **kwargs):
    r"""
    Open(utf8_string, update=0) -> OGRDataSourceShadow *


    Open a vector file as a :py:class:`gdal.Dataset`.
    Equivalent to calling :py:func:`gdal.OpenEx` with the
    :py:const:`gdal.OF_VECTOR` flag.

    Parameters
    ----------
    path : str
        name of the file to open

    Returns
    -------
    Dataset or None
        A dataset or ``None`` on failure.

    Examples
    --------
    >>> from osgeo import ogr
    >>> ogr.GetDriverByName('ESRI Shapefile').GetDescription()
    'ESRI Shapefile'
    >>> ogr.GetDriverByName('GTiff')
    >>>

    """

    _WarnIfUserHasNotSpecifiedIfUsingExceptions()
    from . import gdal


    return _ogr.Open(*args, **kwargs)

def OpenShared(*args, **kwargs):
    r"""
    OpenShared(utf8_string, update=0) -> OGRDataSourceShadow *


    Open a vector file as a :py:class:`gdal.Dataset`. If the file has already been
    opened in the current thread, return a reference to the already-opened
    :py:class:`gdal.Dataset`. Equivalent to calling :py:func:`gdal.OpenEx` with the
    :py:const:`gdal.OF_VECTOR` and :py:const:`gdal.OF_SHARED` flags.

    Parameters
    ----------
    path : str
        name of the file to open

    Returns
    -------
    Dataset or None
        A dataset or ``None`` on failure.


    """

    from . import gdal


    return _ogr.OpenShared(*args, **kwargs)

def GetDriverByName(*args):
    r"""
    GetDriverByName(name) -> OGRDriverShadow *


    Get a vector driver. Like :py:func:`gdal.GetDriverByName`, but
    only returns drivers that handle vector data.

    Parameters
    ----------
    name : str
        name of the driver to fetch

    Returns
    -------
    Driver

    Examples
    --------
    >>> ogr.GetDriverByName('ESRI Shapefile').GetDescription()
    'ESRI Shapefile'

    >>> ogr.GetDriverByName('GTiff')
    >>>

    """

    from . import gdal


    return _ogr.GetDriverByName(*args)

def GetDriver(*args):
    r"""GetDriver(driver_number) -> OGRDriverShadow *"""

    from . import gdal


    return _ogr.GetDriver(*args)

def GeneralCmdLineProcessor(*args):
    r"""GeneralCmdLineProcessor(papszArgv, nOptions=0) -> char **"""

    import os
    for i in range(len(args[0])):
        if isinstance(args[0][i], (os.PathLike, int)):
            args[0][i] = str(args[0][i])


    return _ogr.GeneralCmdLineProcessor(*args)

def TermProgress_nocb(*args, **kwargs):
    r"""TermProgress_nocb(dfProgress, pszMessage=None, pData=None) -> int"""
    return _ogr.TermProgress_nocb(*args, **kwargs)
TermProgress = _ogr.TermProgress


